from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/bottlecaps.scad'}", use_not_include=False)

_sp_specs = OpenSCADConstant('_sp_specs')
_sp_twist = OpenSCADConstant('_sp_twist')
_sp_thread_width = OpenSCADConstant('_sp_thread_width')
class pco1810_neck(OpenSCADObject):
    def __init__(self, wall=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("pco1810_neck", {"wall" : wall, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class pco1810_cap(OpenSCADObject):
    def __init__(self, wall=None, texture=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("pco1810_cap", {"wall" : wall, "texture" : texture, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class pco1881_neck(OpenSCADObject):
    def __init__(self, wall=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("pco1881_neck", {"wall" : wall, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class pco1881_cap(OpenSCADObject):
    def __init__(self, wall=None, texture=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("pco1881_cap", {"wall" : wall, "texture" : texture, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class generic_bottle_neck(OpenSCADObject):
    def __init__(self, wall=None, neck_d=None, id=None, thread_od=None, height=None, support_d=None, pitch=None, round_supp=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("generic_bottle_neck", {"wall" : wall, "neck_d" : neck_d, "id" : id, "thread_od" : thread_od, "height" : height, "support_d" : support_d, "pitch" : pitch, "round_supp" : round_supp, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class generic_bottle_cap(OpenSCADObject):
    def __init__(self, wall=None, texture=None, height=None, thread_od=None, tolerance=None, neck_od=None, flank_angle=None, pitch=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("generic_bottle_cap", {"wall" : wall, "texture" : texture, "height" : height, "thread_od" : thread_od, "tolerance" : tolerance, "neck_od" : neck_od, "flank_angle" : flank_angle, "pitch" : pitch, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class bottle_adapter_neck_to_cap(OpenSCADObject):
    def __init__(self, wall=None, texture=None, cap_wall=None, cap_h=None, cap_thread_od=None, tolerance=None, cap_neck_od=None, cap_neck_id=None, cap_thread_taper=None, cap_thread_pitch=None, neck_d=None, neck_id=None, neck_thread_od=None, neck_h=None, neck_thread_pitch=None, neck_support_od=None, d=None, taper_lead_in=None, **kwargs):
       super().__init__("bottle_adapter_neck_to_cap", {"wall" : wall, "texture" : texture, "cap_wall" : cap_wall, "cap_h" : cap_h, "cap_thread_od" : cap_thread_od, "tolerance" : tolerance, "cap_neck_od" : cap_neck_od, "cap_neck_id" : cap_neck_id, "cap_thread_taper" : cap_thread_taper, "cap_thread_pitch" : cap_thread_pitch, "neck_d" : neck_d, "neck_id" : neck_id, "neck_thread_od" : neck_thread_od, "neck_h" : neck_h, "neck_thread_pitch" : neck_thread_pitch, "neck_support_od" : neck_support_od, "d" : d, "taper_lead_in" : taper_lead_in, **kwargs})

class bottle_adapter_cap_to_cap(OpenSCADObject):
    def __init__(self, wall=None, texture=None, cap_h1=None, cap_thread_od1=None, tolerance=None, cap_neck_od1=None, cap_thread_pitch1=None, cap_h2=None, cap_thread_od2=None, cap_neck_od2=None, cap_thread_pitch2=None, d=None, neck_id1=None, neck_id2=None, taper_lead_in=None, **kwargs):
       super().__init__("bottle_adapter_cap_to_cap", {"wall" : wall, "texture" : texture, "cap_h1" : cap_h1, "cap_thread_od1" : cap_thread_od1, "tolerance" : tolerance, "cap_neck_od1" : cap_neck_od1, "cap_thread_pitch1" : cap_thread_pitch1, "cap_h2" : cap_h2, "cap_thread_od2" : cap_thread_od2, "cap_neck_od2" : cap_neck_od2, "cap_thread_pitch2" : cap_thread_pitch2, "d" : d, "neck_id1" : neck_id1, "neck_id2" : neck_id2, "taper_lead_in" : taper_lead_in, **kwargs})

class bottle_adapter_neck_to_neck(OpenSCADObject):
    def __init__(self, d=None, neck_od1=None, neck_id1=None, thread_od1=None, height1=None, support_od1=None, thread_pitch1=None, neck_od2=None, neck_id2=None, thread_od2=None, height2=None, support_od2=None, pitch2=None, taper_lead_in=None, wall=None, **kwargs):
       super().__init__("bottle_adapter_neck_to_neck", {"d" : d, "neck_od1" : neck_od1, "neck_id1" : neck_id1, "thread_od1" : thread_od1, "height1" : height1, "support_od1" : support_od1, "thread_pitch1" : thread_pitch1, "neck_od2" : neck_od2, "neck_id2" : neck_id2, "thread_od2" : thread_od2, "height2" : height2, "support_od2" : support_od2, "pitch2" : pitch2, "taper_lead_in" : taper_lead_in, "wall" : wall, **kwargs})

class sp_neck(OpenSCADObject):
    def __init__(self, diam=None, type=None, wall=None, id=None, style=None, bead=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("sp_neck", {"diam" : diam, "type" : type, "wall" : wall, "id" : id, "style" : style, "bead" : bead, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class pco1810_neck(OpenSCADObject):
    def __init__(self, wall=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("pco1810_neck", {"wall" : wall, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class pco1810_cap(OpenSCADObject):
    def __init__(self, wall=None, texture=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("pco1810_cap", {"wall" : wall, "texture" : texture, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class pco1881_neck(OpenSCADObject):
    def __init__(self, wall=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("pco1881_neck", {"wall" : wall, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class pco1881_cap(OpenSCADObject):
    def __init__(self, wall=None, texture=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("pco1881_cap", {"wall" : wall, "texture" : texture, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class generic_bottle_neck(OpenSCADObject):
    def __init__(self, neck_d=None, id=None, thread_od=None, height=None, support_d=None, pitch=None, round_supp=None, wall=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("generic_bottle_neck", {"neck_d" : neck_d, "id" : id, "thread_od" : thread_od, "height" : height, "support_d" : support_d, "pitch" : pitch, "round_supp" : round_supp, "wall" : wall, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class generic_bottle_cap(OpenSCADObject):
    def __init__(self, wall=None, texture=None, height=None, thread_od=None, tolerance=None, neck_od=None, flank_angle=None, pitch=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("generic_bottle_cap", {"wall" : wall, "texture" : texture, "height" : height, "thread_od" : thread_od, "tolerance" : tolerance, "neck_od" : neck_od, "flank_angle" : flank_angle, "pitch" : pitch, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class bottle_adapter_neck_to_cap(OpenSCADObject):
    def __init__(self, wall=None, texture=None, cap_wall=None, cap_h=None, cap_thread_od=None, tolerance=None, cap_neck_od=None, cap_neck_id=None, cap_thread_taper=None, cap_thread_pitch=None, neck_d=None, neck_id=None, neck_thread_od=None, neck_h=None, neck_thread_pitch=None, neck_support_od=None, d=None, taper_lead_in=None, **kwargs):
       super().__init__("bottle_adapter_neck_to_cap", {"wall" : wall, "texture" : texture, "cap_wall" : cap_wall, "cap_h" : cap_h, "cap_thread_od" : cap_thread_od, "tolerance" : tolerance, "cap_neck_od" : cap_neck_od, "cap_neck_id" : cap_neck_id, "cap_thread_taper" : cap_thread_taper, "cap_thread_pitch" : cap_thread_pitch, "neck_d" : neck_d, "neck_id" : neck_id, "neck_thread_od" : neck_thread_od, "neck_h" : neck_h, "neck_thread_pitch" : neck_thread_pitch, "neck_support_od" : neck_support_od, "d" : d, "taper_lead_in" : taper_lead_in, **kwargs})

class bottle_adapter_cap_to_cap(OpenSCADObject):
    def __init__(self, wall=None, texture=None, cap_h1=None, cap_thread_od1=None, tolerance=None, cap_neck_od1=None, cap_thread_pitch1=None, cap_h2=None, cap_thread_od2=None, cap_neck_od2=None, cap_thread_pitch2=None, d=None, neck_id1=None, neck_id2=None, taper_lead_in=None, **kwargs):
       super().__init__("bottle_adapter_cap_to_cap", {"wall" : wall, "texture" : texture, "cap_h1" : cap_h1, "cap_thread_od1" : cap_thread_od1, "tolerance" : tolerance, "cap_neck_od1" : cap_neck_od1, "cap_thread_pitch1" : cap_thread_pitch1, "cap_h2" : cap_h2, "cap_thread_od2" : cap_thread_od2, "cap_neck_od2" : cap_neck_od2, "cap_thread_pitch2" : cap_thread_pitch2, "d" : d, "neck_id1" : neck_id1, "neck_id2" : neck_id2, "taper_lead_in" : taper_lead_in, **kwargs})

class bottle_adapter_neck_to_neck(OpenSCADObject):
    def __init__(self, d=None, neck_od1=None, neck_id1=None, thread_od1=None, height1=None, support_od1=None, thread_pitch1=None, neck_od2=None, neck_id2=None, thread_od2=None, height2=None, support_od2=None, pitch2=None, taper_lead_in=None, wall=None, **kwargs):
       super().__init__("bottle_adapter_neck_to_neck", {"d" : d, "neck_od1" : neck_od1, "neck_id1" : neck_id1, "thread_od1" : thread_od1, "height1" : height1, "support_od1" : support_od1, "thread_pitch1" : thread_pitch1, "neck_od2" : neck_od2, "neck_id2" : neck_id2, "thread_od2" : thread_od2, "height2" : height2, "support_od2" : support_od2, "pitch2" : pitch2, "taper_lead_in" : taper_lead_in, "wall" : wall, **kwargs})

class _sp_thread_profile(OpenSCADObject):
    def __init__(self, tpi=None, a=None, S=None, style=None, **kwargs):
       super().__init__("_sp_thread_profile", {"tpi" : tpi, "a" : a, "S" : S, "style" : style, **kwargs})

class sp_neck(OpenSCADObject):
    def __init__(self, diam=None, type=None, wall=None, id=None, style=None, bead=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("sp_neck", {"diam" : diam, "type" : type, "wall" : wall, "id" : id, "style" : style, "bead" : bead, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class sp_diameter(OpenSCADObject):
    def __init__(self, diam=None, type=None, **kwargs):
       super().__init__("sp_diameter", {"diam" : diam, "type" : type, **kwargs})




#! /usr/bin/env python

from solid2 import *

#register all the custom variables you want to use
objects = CustomizerDropdownVariable("objects", 4, [2, 4, 6])
side = CustomizerSliderVariable("side", 4)
cube_pos = CustomizerSliderVariable("cube_pos", [5, 5, 5])
cube_size = CustomizerSliderVariable("cube_size", 5)
customizedText = CustomizerDropdownVariable("text", "customize me!",
                                            ["customize me!", "Thank you!"])

#use scad_inline to use them
scene = scad_inline("""
                    for (i = [1:objects]){
                        translate([2*i*side,0,0]){
                            cube(side);
                        }
                    }
                    """)

scene += translate(cube_pos) (
            cube(cube_size * 2))

scene += translate([0, -20, 0]) (
            text(customizedText))

scad_render_to_file(scene)




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/color.scad'}", use_not_include=False)

class recolor(OpenSCADObject):
    def __init__(self, c=None, **kwargs):
       super().__init__("recolor", {"c" : c, **kwargs})

class color_this(OpenSCADObject):
    def __init__(self, c=None, **kwargs):
       super().__init__("color_this", {"c" : c, **kwargs})

class rainbow(OpenSCADObject):
    def __init__(self, list=None, stride=None, maxhues=None, shuffle=None, seed=None, **kwargs):
       super().__init__("rainbow", {"list" : list, "stride" : stride, "maxhues" : maxhues, "shuffle" : shuffle, "seed" : seed, **kwargs})

class hsl(OpenSCADObject):
    def __init__(self, h=None, s=None, l=None, a=None, **kwargs):
       super().__init__("hsl", {"h" : h, "s" : s, "l" : l, "a" : a, **kwargs})

class hsv(OpenSCADObject):
    def __init__(self, h=None, s=None, v=None, a=None, **kwargs):
       super().__init__("hsv", {"h" : h, "s" : s, "v" : v, "a" : a, **kwargs})

class hsl(OpenSCADObject):
    def __init__(self, h=None, s=None, l=None, a=None, **kwargs):
       super().__init__("hsl", {"h" : h, "s" : s, "l" : l, "a" : a, **kwargs})

class hsv(OpenSCADObject):
    def __init__(self, h=None, s=None, v=None, a=None, **kwargs):
       super().__init__("hsv", {"h" : h, "s" : s, "v" : v, "a" : a, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/shapes2d.scad'}", use_not_include=False)

class square(OpenSCADObject):
    def __init__(self, size=None, center=None, anchor=None, spin=None, **kwargs):
       super().__init__("square", {"size" : size, "center" : center, "anchor" : anchor, "spin" : spin, **kwargs})

class rect(OpenSCADObject):
    def __init__(self, size=None, rounding=None, atype=None, chamfer=None, anchor=None, spin=None, **kwargs):
       super().__init__("rect", {"size" : size, "rounding" : rounding, "atype" : atype, "chamfer" : chamfer, "anchor" : anchor, "spin" : spin, **kwargs})

class circle(OpenSCADObject):
    def __init__(self, r=None, d=None, points=None, corner=None, anchor=None, spin=None, **kwargs):
       super().__init__("circle", {"r" : r, "d" : d, "points" : points, "corner" : corner, "anchor" : anchor, "spin" : spin, **kwargs})

class ellipse(OpenSCADObject):
    def __init__(self, r=None, d=None, realign=None, circum=None, uniform=None, anchor=None, spin=None, **kwargs):
       super().__init__("ellipse", {"r" : r, "d" : d, "realign" : realign, "circum" : circum, "uniform" : uniform, "anchor" : anchor, "spin" : spin, **kwargs})

class regular_ngon(OpenSCADObject):
    def __init__(self, n=None, r=None, d=None, _or=None, od=None, ir=None, id=None, side=None, rounding=None, realign=None, align_tip=None, align_side=None, anchor=None, spin=None, **kwargs):
       super().__init__("regular_ngon", {"n" : n, "r" : r, "d" : d, "_or" : _or, "od" : od, "ir" : ir, "id" : id, "side" : side, "rounding" : rounding, "realign" : realign, "align_tip" : align_tip, "align_side" : align_side, "anchor" : anchor, "spin" : spin, **kwargs})

class pentagon(OpenSCADObject):
    def __init__(self, r=None, d=None, _or=None, od=None, ir=None, id=None, side=None, rounding=None, realign=None, align_tip=None, align_side=None, anchor=None, spin=None, **kwargs):
       super().__init__("pentagon", {"r" : r, "d" : d, "_or" : _or, "od" : od, "ir" : ir, "id" : id, "side" : side, "rounding" : rounding, "realign" : realign, "align_tip" : align_tip, "align_side" : align_side, "anchor" : anchor, "spin" : spin, **kwargs})

class hexagon(OpenSCADObject):
    def __init__(self, r=None, d=None, _or=None, od=None, ir=None, id=None, side=None, rounding=None, realign=None, align_tip=None, align_side=None, anchor=None, spin=None, **kwargs):
       super().__init__("hexagon", {"r" : r, "d" : d, "_or" : _or, "od" : od, "ir" : ir, "id" : id, "side" : side, "rounding" : rounding, "realign" : realign, "align_tip" : align_tip, "align_side" : align_side, "anchor" : anchor, "spin" : spin, **kwargs})

class octagon(OpenSCADObject):
    def __init__(self, r=None, d=None, _or=None, od=None, ir=None, id=None, side=None, rounding=None, realign=None, align_tip=None, align_side=None, anchor=None, spin=None, **kwargs):
       super().__init__("octagon", {"r" : r, "d" : d, "_or" : _or, "od" : od, "ir" : ir, "id" : id, "side" : side, "rounding" : rounding, "realign" : realign, "align_tip" : align_tip, "align_side" : align_side, "anchor" : anchor, "spin" : spin, **kwargs})

class right_triangle(OpenSCADObject):
    def __init__(self, size=None, center=None, anchor=None, spin=None, **kwargs):
       super().__init__("right_triangle", {"size" : size, "center" : center, "anchor" : anchor, "spin" : spin, **kwargs})

class trapezoid(OpenSCADObject):
    def __init__(self, h=None, w1=None, w2=None, angle=None, shift=None, chamfer=None, rounding=None, flip=None, anchor=None, spin=None, **kwargs):
       super().__init__("trapezoid", {"h" : h, "w1" : w1, "w2" : w2, "angle" : angle, "shift" : shift, "chamfer" : chamfer, "rounding" : rounding, "flip" : flip, "anchor" : anchor, "spin" : spin, **kwargs})

class star(OpenSCADObject):
    def __init__(self, n=None, r=None, ir=None, d=None, _or=None, od=None, id=None, step=None, realign=None, align_tip=None, align_pit=None, anchor=None, spin=None, atype=None, **kwargs):
       super().__init__("star", {"n" : n, "r" : r, "ir" : ir, "d" : d, "_or" : _or, "od" : od, "id" : id, "step" : step, "realign" : realign, "align_tip" : align_tip, "align_pit" : align_pit, "anchor" : anchor, "spin" : spin, "atype" : atype, **kwargs})

class jittered_poly(OpenSCADObject):
    def __init__(self, path=None, dist=None, **kwargs):
       super().__init__("jittered_poly", {"path" : path, "dist" : dist, **kwargs})

class teardrop2d(OpenSCADObject):
    def __init__(self, r=None, ang=None, cap_h=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("teardrop2d", {"r" : r, "ang" : ang, "cap_h" : cap_h, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class egg(OpenSCADObject):
    def __init__(self, length=None, r1=None, r2=None, R=None, d1=None, d2=None, D=None, anchor=None, spin=None, **kwargs):
       super().__init__("egg", {"length" : length, "r1" : r1, "r2" : r2, "R" : R, "d1" : d1, "d2" : d2, "D" : D, "anchor" : anchor, "spin" : spin, **kwargs})

class glued_circles(OpenSCADObject):
    def __init__(self, r=None, spread=None, tangent=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("glued_circles", {"r" : r, "spread" : spread, "tangent" : tangent, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class supershape(OpenSCADObject):
    def __init__(self, step=None, m1=None, m2=None, n1=None, n2=None, n3=None, a=None, b=None, r=None, d=None, anchor=None, spin=None, atype=None, **kwargs):
       super().__init__("supershape", {"step" : step, "m1" : m1, "m2" : m2, "n1" : n1, "n2" : n2, "n3" : n3, "a" : a, "b" : b, "r" : r, "d" : d, "anchor" : anchor, "spin" : spin, "atype" : atype, **kwargs})

class reuleaux_polygon(OpenSCADObject):
    def __init__(self, n=None, r=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("reuleaux_polygon", {"n" : n, "r" : r, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class text(OpenSCADObject):
    def __init__(self, text=None, size=None, font=None, halign=None, valign=None, spacing=None, direction=None, language=None, script=None, anchor=None, spin=None, **kwargs):
       super().__init__("text", {"text" : text, "size" : size, "font" : font, "halign" : halign, "valign" : valign, "spacing" : spacing, "direction" : direction, "language" : language, "script" : script, "anchor" : anchor, "spin" : spin, **kwargs})

class round2d(OpenSCADObject):
    def __init__(self, r=None, _or=None, ir=None, **kwargs):
       super().__init__("round2d", {"r" : r, "_or" : _or, "ir" : ir, **kwargs})

class shell2d(OpenSCADObject):
    def __init__(self, thickness=None, _or=None, ir=None, **kwargs):
       super().__init__("shell2d", {"thickness" : thickness, "_or" : _or, "ir" : ir, **kwargs})

class square(OpenSCADObject):
    def __init__(self, size=None, center=None, anchor=None, spin=None, **kwargs):
       super().__init__("square", {"size" : size, "center" : center, "anchor" : anchor, "spin" : spin, **kwargs})

class rect(OpenSCADObject):
    def __init__(self, size=None, rounding=None, chamfer=None, atype=None, anchor=None, spin=None, **kwargs):
       super().__init__("rect", {"size" : size, "rounding" : rounding, "chamfer" : chamfer, "atype" : atype, "anchor" : anchor, "spin" : spin, **kwargs})

class circle(OpenSCADObject):
    def __init__(self, r=None, d=None, points=None, corner=None, anchor=None, spin=None, **kwargs):
       super().__init__("circle", {"r" : r, "d" : d, "points" : points, "corner" : corner, "anchor" : anchor, "spin" : spin, **kwargs})

class _ellipse_refine(OpenSCADObject):
    def __init__(self, a=None, b=None, N=None, _theta=None, **kwargs):
       super().__init__("_ellipse_refine", {"a" : a, "b" : b, "N" : N, "_theta" : _theta, **kwargs})

class _ellipse_refine_realign(OpenSCADObject):
    def __init__(self, a=None, b=None, N=None, _theta=None, i=None, **kwargs):
       super().__init__("_ellipse_refine_realign", {"a" : a, "b" : b, "N" : N, "_theta" : _theta, "i" : i, **kwargs})

class ellipse(OpenSCADObject):
    def __init__(self, r=None, d=None, realign=None, circum=None, uniform=None, anchor=None, spin=None, **kwargs):
       super().__init__("ellipse", {"r" : r, "d" : d, "realign" : realign, "circum" : circum, "uniform" : uniform, "anchor" : anchor, "spin" : spin, **kwargs})

class regular_ngon(OpenSCADObject):
    def __init__(self, n=None, r=None, d=None, _or=None, od=None, ir=None, id=None, side=None, rounding=None, realign=None, align_tip=None, align_side=None, anchor=None, spin=None, _mat=None, _anchs=None, **kwargs):
       super().__init__("regular_ngon", {"n" : n, "r" : r, "d" : d, "_or" : _or, "od" : od, "ir" : ir, "id" : id, "side" : side, "rounding" : rounding, "realign" : realign, "align_tip" : align_tip, "align_side" : align_side, "anchor" : anchor, "spin" : spin, "_mat" : _mat, "_anchs" : _anchs, **kwargs})

class pentagon(OpenSCADObject):
    def __init__(self, r=None, d=None, _or=None, od=None, ir=None, id=None, side=None, rounding=None, realign=None, align_tip=None, align_side=None, anchor=None, spin=None, **kwargs):
       super().__init__("pentagon", {"r" : r, "d" : d, "_or" : _or, "od" : od, "ir" : ir, "id" : id, "side" : side, "rounding" : rounding, "realign" : realign, "align_tip" : align_tip, "align_side" : align_side, "anchor" : anchor, "spin" : spin, **kwargs})

class hexagon(OpenSCADObject):
    def __init__(self, r=None, d=None, _or=None, od=None, ir=None, id=None, side=None, rounding=None, realign=None, align_tip=None, align_side=None, anchor=None, spin=None, **kwargs):
       super().__init__("hexagon", {"r" : r, "d" : d, "_or" : _or, "od" : od, "ir" : ir, "id" : id, "side" : side, "rounding" : rounding, "realign" : realign, "align_tip" : align_tip, "align_side" : align_side, "anchor" : anchor, "spin" : spin, **kwargs})

class octagon(OpenSCADObject):
    def __init__(self, r=None, d=None, _or=None, od=None, ir=None, id=None, side=None, rounding=None, realign=None, align_tip=None, align_side=None, anchor=None, spin=None, **kwargs):
       super().__init__("octagon", {"r" : r, "d" : d, "_or" : _or, "od" : od, "ir" : ir, "id" : id, "side" : side, "rounding" : rounding, "realign" : realign, "align_tip" : align_tip, "align_side" : align_side, "anchor" : anchor, "spin" : spin, **kwargs})

class right_triangle(OpenSCADObject):
    def __init__(self, size=None, center=None, anchor=None, spin=None, **kwargs):
       super().__init__("right_triangle", {"size" : size, "center" : center, "anchor" : anchor, "spin" : spin, **kwargs})

class trapezoid(OpenSCADObject):
    def __init__(self, h=None, w1=None, w2=None, angle=None, shift=None, chamfer=None, rounding=None, flip=None, anchor=None, spin=None, **kwargs):
       super().__init__("trapezoid", {"h" : h, "w1" : w1, "w2" : w2, "angle" : angle, "shift" : shift, "chamfer" : chamfer, "rounding" : rounding, "flip" : flip, "anchor" : anchor, "spin" : spin, **kwargs})

class star(OpenSCADObject):
    def __init__(self, n=None, r=None, ir=None, d=None, _or=None, od=None, id=None, step=None, realign=None, align_tip=None, align_pit=None, anchor=None, spin=None, atype=None, _mat=None, _anchs=None, **kwargs):
       super().__init__("star", {"n" : n, "r" : r, "ir" : ir, "d" : d, "_or" : _or, "od" : od, "id" : id, "step" : step, "realign" : realign, "align_tip" : align_tip, "align_pit" : align_pit, "anchor" : anchor, "spin" : spin, "atype" : atype, "_mat" : _mat, "_anchs" : _anchs, **kwargs})

class _path_add_jitter(OpenSCADObject):
    def __init__(self, path=None, dist=None, closed=None, **kwargs):
       super().__init__("_path_add_jitter", {"path" : path, "dist" : dist, "closed" : closed, **kwargs})

class teardrop2d(OpenSCADObject):
    def __init__(self, r=None, ang=None, cap_h=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("teardrop2d", {"r" : r, "ang" : ang, "cap_h" : cap_h, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class egg(OpenSCADObject):
    def __init__(self, length=None, r1=None, r2=None, R=None, d1=None, d2=None, D=None, anchor=None, spin=None, **kwargs):
       super().__init__("egg", {"length" : length, "r1" : r1, "r2" : r2, "R" : R, "d1" : d1, "d2" : d2, "D" : D, "anchor" : anchor, "spin" : spin, **kwargs})

class glued_circles(OpenSCADObject):
    def __init__(self, r=None, spread=None, tangent=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("glued_circles", {"r" : r, "spread" : spread, "tangent" : tangent, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class _superformula(OpenSCADObject):
    def __init__(self, theta=None, m1=None, m2=None, n1=None, n2=None, n3=None, a=None, b=None, **kwargs):
       super().__init__("_superformula", {"theta" : theta, "m1" : m1, "m2" : m2, "n1" : n1, "n2" : n2, "n3" : n3, "a" : a, "b" : b, **kwargs})

class supershape(OpenSCADObject):
    def __init__(self, step=None, m1=None, m2=None, n1=None, n2=None, n3=None, a=None, b=None, r=None, d=None, anchor=None, spin=None, atype=None, **kwargs):
       super().__init__("supershape", {"step" : step, "m1" : m1, "m2" : m2, "n1" : n1, "n2" : n2, "n3" : n3, "a" : a, "b" : b, "r" : r, "d" : d, "anchor" : anchor, "spin" : spin, "atype" : atype, **kwargs})

class reuleaux_polygon(OpenSCADObject):
    def __init__(self, n=None, r=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("reuleaux_polygon", {"n" : n, "r" : r, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/strings.scad'}", use_not_include=False)

class substr(OpenSCADObject):
    def __init__(self, str=None, pos=None, len=None, **kwargs):
       super().__init__("substr", {"str" : str, "pos" : pos, "len" : len, **kwargs})

class _substr(OpenSCADObject):
    def __init__(self, str=None, pos=None, len=None, substr=None, **kwargs):
       super().__init__("_substr", {"str" : str, "pos" : pos, "len" : len, "substr" : substr, **kwargs})

class suffix(OpenSCADObject):
    def __init__(self, str=None, len=None, **kwargs):
       super().__init__("suffix", {"str" : str, "len" : len, **kwargs})

class str_find(OpenSCADObject):
    def __init__(self, str=None, pattern=None, start=None, last=None, all=None, **kwargs):
       super().__init__("str_find", {"str" : str, "pattern" : pattern, "start" : start, "last" : last, "all" : all, **kwargs})

class _str_find_first(OpenSCADObject):
    def __init__(self, str=None, pattern=None, max_sindex=None, sindex=None, **kwargs):
       super().__init__("_str_find_first", {"str" : str, "pattern" : pattern, "max_sindex" : max_sindex, "sindex" : sindex, **kwargs})

class _str_find_last(OpenSCADObject):
    def __init__(self, str=None, pattern=None, sindex=None, **kwargs):
       super().__init__("_str_find_last", {"str" : str, "pattern" : pattern, "sindex" : sindex, **kwargs})

class _str_find_all(OpenSCADObject):
    def __init__(self, str=None, pattern=None, **kwargs):
       super().__init__("_str_find_all", {"str" : str, "pattern" : pattern, **kwargs})

class _str_cmp(OpenSCADObject):
    def __init__(self, str=None, sindex=None, pattern=None, **kwargs):
       super().__init__("_str_cmp", {"str" : str, "sindex" : sindex, "pattern" : pattern, **kwargs})

class _str_cmp_recurse(OpenSCADObject):
    def __init__(self, str=None, sindex=None, pattern=None, plen=None, pindex=None, **kwargs):
       super().__init__("_str_cmp_recurse", {"str" : str, "sindex" : sindex, "pattern" : pattern, "plen" : plen, "pindex" : pindex, **kwargs})

class starts_with(OpenSCADObject):
    def __init__(self, str=None, pattern=None, **kwargs):
       super().__init__("starts_with", {"str" : str, "pattern" : pattern, **kwargs})

class ends_with(OpenSCADObject):
    def __init__(self, str=None, pattern=None, **kwargs):
       super().__init__("ends_with", {"str" : str, "pattern" : pattern, **kwargs})

class str_split(OpenSCADObject):
    def __init__(self, str=None, sep=None, keep_nulls=None, **kwargs):
       super().__init__("str_split", {"str" : str, "sep" : sep, "keep_nulls" : keep_nulls, **kwargs})

class _str_split_recurse(OpenSCADObject):
    def __init__(self, str=None, sep=None, i=None, result=None, **kwargs):
       super().__init__("_str_split_recurse", {"str" : str, "sep" : sep, "i" : i, "result" : result, **kwargs})

class _remove_empty_strs(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("_remove_empty_strs", {"list" : list, **kwargs})

class str_join(OpenSCADObject):
    def __init__(self, list=None, sep=None, _i=None, _result=None, **kwargs):
       super().__init__("str_join", {"list" : list, "sep" : sep, "_i" : _i, "_result" : _result, **kwargs})

class _str_count_leading(OpenSCADObject):
    def __init__(self, s=None, c=None, _i=None, **kwargs):
       super().__init__("_str_count_leading", {"s" : s, "c" : c, "_i" : _i, **kwargs})

class _str_count_trailing(OpenSCADObject):
    def __init__(self, s=None, c=None, _i=None, **kwargs):
       super().__init__("_str_count_trailing", {"s" : s, "c" : c, "_i" : _i, **kwargs})

class str_strip(OpenSCADObject):
    def __init__(self, s=None, c=None, start=None, end=None, **kwargs):
       super().__init__("str_strip", {"s" : s, "c" : c, "start" : start, "end" : end, **kwargs})

class str_pad(OpenSCADObject):
    def __init__(self, str=None, length=None, char=None, left=None, **kwargs):
       super().__init__("str_pad", {"str" : str, "length" : length, "char" : char, "left" : left, **kwargs})

class str_replace_char(OpenSCADObject):
    def __init__(self, str=None, char=None, replace=None, **kwargs):
       super().__init__("str_replace_char", {"str" : str, "char" : char, "replace" : replace, **kwargs})

class downcase(OpenSCADObject):
    def __init__(self, str=None, **kwargs):
       super().__init__("downcase", {"str" : str, **kwargs})

class upcase(OpenSCADObject):
    def __init__(self, str=None, **kwargs):
       super().__init__("upcase", {"str" : str, **kwargs})

class rand_str(OpenSCADObject):
    def __init__(self, n=None, charset=None, seed=None, **kwargs):
       super().__init__("rand_str", {"n" : n, "charset" : charset, "seed" : seed, **kwargs})

class parse_int(OpenSCADObject):
    def __init__(self, str=None, base=None, **kwargs):
       super().__init__("parse_int", {"str" : str, "base" : base, **kwargs})

class _parse_int_recurse(OpenSCADObject):
    def __init__(self, str=None, base=None, i=None, **kwargs):
       super().__init__("_parse_int_recurse", {"str" : str, "base" : base, "i" : i, **kwargs})

class parse_float(OpenSCADObject):
    def __init__(self, str=None, **kwargs):
       super().__init__("parse_float", {"str" : str, **kwargs})

class parse_frac(OpenSCADObject):
    def __init__(self, str=None, mixed=None, improper=None, signed=None, **kwargs):
       super().__init__("parse_frac", {"str" : str, "mixed" : mixed, "improper" : improper, "signed" : signed, **kwargs})

class parse_num(OpenSCADObject):
    def __init__(self, str=None, **kwargs):
       super().__init__("parse_num", {"str" : str, **kwargs})

class format_int(OpenSCADObject):
    def __init__(self, i=None, mindigits=None, **kwargs):
       super().__init__("format_int", {"i" : i, "mindigits" : mindigits, **kwargs})

class format_fixed(OpenSCADObject):
    def __init__(self, f=None, digits=None, **kwargs):
       super().__init__("format_fixed", {"f" : f, "digits" : digits, **kwargs})

class format_float(OpenSCADObject):
    def __init__(self, f=None, sig=None, **kwargs):
       super().__init__("format_float", {"f" : f, "sig" : sig, **kwargs})

class _format_matrix(OpenSCADObject):
    def __init__(self, M=None, sig=None, sep=None, eps=None, **kwargs):
       super().__init__("_format_matrix", {"M" : M, "sig" : sig, "sep" : sep, "eps" : eps, **kwargs})

class format(OpenSCADObject):
    def __init__(self, fmt=None, vals=None, **kwargs):
       super().__init__("format", {"fmt" : fmt, "vals" : vals, **kwargs})

class is_lower(OpenSCADObject):
    def __init__(self, s=None, **kwargs):
       super().__init__("is_lower", {"s" : s, **kwargs})

class is_upper(OpenSCADObject):
    def __init__(self, s=None, **kwargs):
       super().__init__("is_upper", {"s" : s, **kwargs})

class is_digit(OpenSCADObject):
    def __init__(self, s=None, **kwargs):
       super().__init__("is_digit", {"s" : s, **kwargs})

class is_hexdigit(OpenSCADObject):
    def __init__(self, s=None, **kwargs):
       super().__init__("is_hexdigit", {"s" : s, **kwargs})

class is_letter(OpenSCADObject):
    def __init__(self, s=None, **kwargs):
       super().__init__("is_letter", {"s" : s, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/distributors.scad'}", use_not_include=False)

class move_copies(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("move_copies", {"a" : a, **kwargs})

class line_of(OpenSCADObject):
    def __init__(self, spacing=None, n=None, l=None, p1=None, p2=None, **kwargs):
       super().__init__("line_of", {"spacing" : spacing, "n" : n, "l" : l, "p1" : p1, "p2" : p2, **kwargs})

class xcopies(OpenSCADObject):
    def __init__(self, spacing=None, n=None, l=None, sp=None, **kwargs):
       super().__init__("xcopies", {"spacing" : spacing, "n" : n, "l" : l, "sp" : sp, **kwargs})

class ycopies(OpenSCADObject):
    def __init__(self, spacing=None, n=None, l=None, sp=None, **kwargs):
       super().__init__("ycopies", {"spacing" : spacing, "n" : n, "l" : l, "sp" : sp, **kwargs})

class zcopies(OpenSCADObject):
    def __init__(self, spacing=None, n=None, l=None, sp=None, **kwargs):
       super().__init__("zcopies", {"spacing" : spacing, "n" : n, "l" : l, "sp" : sp, **kwargs})

class grid2d(OpenSCADObject):
    def __init__(self, spacing=None, n=None, size=None, stagger=None, inside=None, nonzero=None, **kwargs):
       super().__init__("grid2d", {"spacing" : spacing, "n" : n, "size" : size, "stagger" : stagger, "inside" : inside, "nonzero" : nonzero, **kwargs})

class rot_copies(OpenSCADObject):
    def __init__(self, rots=None, v=None, cp=None, n=None, sa=None, offset=None, delta=None, subrot=None, **kwargs):
       super().__init__("rot_copies", {"rots" : rots, "v" : v, "cp" : cp, "n" : n, "sa" : sa, "offset" : offset, "delta" : delta, "subrot" : subrot, **kwargs})

class xrot_copies(OpenSCADObject):
    def __init__(self, rots=None, cp=None, n=None, sa=None, r=None, d=None, subrot=None, **kwargs):
       super().__init__("xrot_copies", {"rots" : rots, "cp" : cp, "n" : n, "sa" : sa, "r" : r, "d" : d, "subrot" : subrot, **kwargs})

class yrot_copies(OpenSCADObject):
    def __init__(self, rots=None, cp=None, n=None, sa=None, r=None, d=None, subrot=None, **kwargs):
       super().__init__("yrot_copies", {"rots" : rots, "cp" : cp, "n" : n, "sa" : sa, "r" : r, "d" : d, "subrot" : subrot, **kwargs})

class zrot_copies(OpenSCADObject):
    def __init__(self, rots=None, cp=None, n=None, sa=None, r=None, d=None, subrot=None, **kwargs):
       super().__init__("zrot_copies", {"rots" : rots, "cp" : cp, "n" : n, "sa" : sa, "r" : r, "d" : d, "subrot" : subrot, **kwargs})

class arc_of(OpenSCADObject):
    def __init__(self, n=None, r=None, rx=None, ry=None, d=None, dx=None, dy=None, sa=None, ea=None, rot=None, **kwargs):
       super().__init__("arc_of", {"n" : n, "r" : r, "rx" : rx, "ry" : ry, "d" : d, "dx" : dx, "dy" : dy, "sa" : sa, "ea" : ea, "rot" : rot, **kwargs})

class ovoid_spread(OpenSCADObject):
    def __init__(self, n=None, r=None, d=None, cone_ang=None, scale=None, perp=None, **kwargs):
       super().__init__("ovoid_spread", {"n" : n, "r" : r, "d" : d, "cone_ang" : cone_ang, "scale" : scale, "perp" : perp, **kwargs})

class path_spread(OpenSCADObject):
    def __init__(self, path=None, n=None, spacing=None, sp=None, rotate_children=None, closed=None, **kwargs):
       super().__init__("path_spread", {"path" : path, "n" : n, "spacing" : spacing, "sp" : sp, "rotate_children" : rotate_children, "closed" : closed, **kwargs})

class mirror_copy(OpenSCADObject):
    def __init__(self, v=None, offset=None, cp=None, **kwargs):
       super().__init__("mirror_copy", {"v" : v, "offset" : offset, "cp" : cp, **kwargs})

class xflip_copy(OpenSCADObject):
    def __init__(self, offset=None, x=None, **kwargs):
       super().__init__("xflip_copy", {"offset" : offset, "x" : x, **kwargs})

class yflip_copy(OpenSCADObject):
    def __init__(self, offset=None, y=None, **kwargs):
       super().__init__("yflip_copy", {"offset" : offset, "y" : y, **kwargs})

class zflip_copy(OpenSCADObject):
    def __init__(self, offset=None, z=None, **kwargs):
       super().__init__("zflip_copy", {"offset" : offset, "z" : z, **kwargs})

class distribute(OpenSCADObject):
    def __init__(self, spacing=None, sizes=None, dir=None, l=None, **kwargs):
       super().__init__("distribute", {"spacing" : spacing, "sizes" : sizes, "dir" : dir, "l" : l, **kwargs})

class xdistribute(OpenSCADObject):
    def __init__(self, spacing=None, sizes=None, l=None, **kwargs):
       super().__init__("xdistribute", {"spacing" : spacing, "sizes" : sizes, "l" : l, **kwargs})

class ydistribute(OpenSCADObject):
    def __init__(self, spacing=None, sizes=None, l=None, **kwargs):
       super().__init__("ydistribute", {"spacing" : spacing, "sizes" : sizes, "l" : l, **kwargs})

class zdistribute(OpenSCADObject):
    def __init__(self, spacing=None, sizes=None, l=None, **kwargs):
       super().__init__("zdistribute", {"spacing" : spacing, "sizes" : sizes, "l" : l, **kwargs})

class line_of(OpenSCADObject):
    def __init__(self, spacing=None, n=None, l=None, p1=None, p2=None, **kwargs):
       super().__init__("line_of", {"spacing" : spacing, "n" : n, "l" : l, "p1" : p1, "p2" : p2, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/utility.scad'}", use_not_include=False)

class no_children(OpenSCADObject):
    def __init__(self, count=None, **kwargs):
       super().__init__("no_children", {"count" : count, **kwargs})

class req_children(OpenSCADObject):
    def __init__(self, count=None, **kwargs):
       super().__init__("req_children", {"count" : count, **kwargs})

class no_module(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("no_module", {**kwargs})

class assert_approx(OpenSCADObject):
    def __init__(self, got=None, expected=None, info=None, **kwargs):
       super().__init__("assert_approx", {"got" : got, "expected" : expected, "info" : info, **kwargs})

class assert_equal(OpenSCADObject):
    def __init__(self, got=None, expected=None, info=None, **kwargs):
       super().__init__("assert_equal", {"got" : got, "expected" : expected, "info" : info, **kwargs})

class shape_compare(OpenSCADObject):
    def __init__(self, eps=None, **kwargs):
       super().__init__("shape_compare", {"eps" : eps, **kwargs})

class typeof(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("typeof", {"x" : x, **kwargs})

class is_type(OpenSCADObject):
    def __init__(self, x=None, types=None, **kwargs):
       super().__init__("is_type", {"x" : x, "types" : types, **kwargs})

class is_def(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_def", {"x" : x, **kwargs})

class is_str(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_str", {"x" : x, **kwargs})

class is_int(OpenSCADObject):
    def __init__(self, n=None, **kwargs):
       super().__init__("is_int", {"n" : n, **kwargs})

class is_integer(OpenSCADObject):
    def __init__(self, n=None, **kwargs):
       super().__init__("is_integer", {"n" : n, **kwargs})

class all_integer(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("all_integer", {"x" : x, **kwargs})

class is_nan(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_nan", {"x" : x, **kwargs})

class is_finite(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_finite", {"x" : x, **kwargs})

class is_range(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_range", {"x" : x, **kwargs})

class valid_range(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("valid_range", {"x" : x, **kwargs})

class is_func(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_func", {"x" : x, **kwargs})

class is_consistent(OpenSCADObject):
    def __init__(self, list=None, pattern=None, **kwargs):
       super().__init__("is_consistent", {"list" : list, "pattern" : pattern, **kwargs})

class _list_pattern(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("_list_pattern", {"list" : list, **kwargs})

class same_shape(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("same_shape", {"a" : a, "b" : b, **kwargs})

class is_bool_list(OpenSCADObject):
    def __init__(self, list=None, length=None, **kwargs):
       super().__init__("is_bool_list", {"list" : list, "length" : length, **kwargs})

class any(OpenSCADObject):
    def __init__(self, l=None, func=None, **kwargs):
       super().__init__("any", {"l" : l, "func" : func, **kwargs})

class _any_func(OpenSCADObject):
    def __init__(self, l=None, func=None, i=None, out=None, **kwargs):
       super().__init__("_any_func", {"l" : l, "func" : func, "i" : i, "out" : out, **kwargs})

class _any_bool(OpenSCADObject):
    def __init__(self, l=None, i=None, out=None, **kwargs):
       super().__init__("_any_bool", {"l" : l, "i" : i, "out" : out, **kwargs})

class all(OpenSCADObject):
    def __init__(self, l=None, func=None, **kwargs):
       super().__init__("all", {"l" : l, "func" : func, **kwargs})

class _all_func(OpenSCADObject):
    def __init__(self, l=None, func=None, i=None, out=None, **kwargs):
       super().__init__("_all_func", {"l" : l, "func" : func, "i" : i, "out" : out, **kwargs})

class _all_bool(OpenSCADObject):
    def __init__(self, l=None, i=None, out=None, **kwargs):
       super().__init__("_all_bool", {"l" : l, "i" : i, "out" : out, **kwargs})

class num_true(OpenSCADObject):
    def __init__(self, l=None, func=None, **kwargs):
       super().__init__("num_true", {"l" : l, "func" : func, **kwargs})

class default(OpenSCADObject):
    def __init__(self, v=None, dflt=None, **kwargs):
       super().__init__("default", {"v" : v, "dflt" : dflt, **kwargs})

class first_defined(OpenSCADObject):
    def __init__(self, v=None, recursive=None, _i=None, **kwargs):
       super().__init__("first_defined", {"v" : v, "recursive" : recursive, "_i" : _i, **kwargs})

class one_defined(OpenSCADObject):
    def __init__(self, vals=None, names=None, dflt=None, **kwargs):
       super().__init__("one_defined", {"vals" : vals, "names" : names, "dflt" : dflt, **kwargs})

class num_defined(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("num_defined", {"v" : v, **kwargs})

class any_defined(OpenSCADObject):
    def __init__(self, v=None, recursive=None, **kwargs):
       super().__init__("any_defined", {"v" : v, "recursive" : recursive, **kwargs})

class all_defined(OpenSCADObject):
    def __init__(self, v=None, recursive=None, **kwargs):
       super().__init__("all_defined", {"v" : v, "recursive" : recursive, **kwargs})

class u_add(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("u_add", {"a" : a, "b" : b, **kwargs})

class u_sub(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("u_sub", {"a" : a, "b" : b, **kwargs})

class u_mul(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("u_mul", {"a" : a, "b" : b, **kwargs})

class u_div(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("u_div", {"a" : a, "b" : b, **kwargs})

class get_anchor(OpenSCADObject):
    def __init__(self, anchor=None, center=None, uncentered=None, dflt=None, **kwargs):
       super().__init__("get_anchor", {"anchor" : anchor, "center" : center, "uncentered" : uncentered, "dflt" : dflt, **kwargs})

class get_radius(OpenSCADObject):
    def __init__(self, r1=None, r2=None, r=None, d1=None, d2=None, d=None, dflt=None, **kwargs):
       super().__init__("get_radius", {"r1" : r1, "r2" : r2, "r" : r, "d1" : d1, "d2" : d2, "d" : d, "dflt" : dflt, **kwargs})

class scalar_vec3(OpenSCADObject):
    def __init__(self, v=None, dflt=None, **kwargs):
       super().__init__("scalar_vec3", {"v" : v, "dflt" : dflt, **kwargs})

class segs(OpenSCADObject):
    def __init__(self, r=None, **kwargs):
       super().__init__("segs", {"r" : r, **kwargs})

class no_function(OpenSCADObject):
    def __init__(self, name=None, **kwargs):
       super().__init__("no_function", {"name" : name, **kwargs})

class _valstr(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("_valstr", {"x" : x, **kwargs})

class looping(OpenSCADObject):
    def __init__(self, state=None, **kwargs):
       super().__init__("looping", {"state" : state, **kwargs})

class loop_while(OpenSCADObject):
    def __init__(self, state=None, _continue=None, **kwargs):
       super().__init__("loop_while", {"state" : state, "_continue" : _continue, **kwargs})

class loop_done(OpenSCADObject):
    def __init__(self, state=None, **kwargs):
       super().__init__("loop_done", {"state" : state, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/masks3d.scad'}", use_not_include=False)

class chamfer_edge_mask(OpenSCADObject):
    def __init__(self, l=None, chamfer=None, excess=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("chamfer_edge_mask", {"l" : l, "chamfer" : chamfer, "excess" : excess, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class chamfer_corner_mask(OpenSCADObject):
    def __init__(self, chamfer=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("chamfer_corner_mask", {"chamfer" : chamfer, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class chamfer_cylinder_mask(OpenSCADObject):
    def __init__(self, r=None, chamfer=None, d=None, ang=None, from_end=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("chamfer_cylinder_mask", {"r" : r, "chamfer" : chamfer, "d" : d, "ang" : ang, "from_end" : from_end, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rounding_edge_mask(OpenSCADObject):
    def __init__(self, l=None, r=None, r1=None, r2=None, d=None, d1=None, d2=None, excess=None, anchor=None, spin=None, orient=None, h=None, **kwargs):
       super().__init__("rounding_edge_mask", {"l" : l, "r" : r, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "excess" : excess, "anchor" : anchor, "spin" : spin, "orient" : orient, "h" : h, **kwargs})

class rounding_corner_mask(OpenSCADObject):
    def __init__(self, r=None, d=None, style=None, excess=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rounding_corner_mask", {"r" : r, "d" : d, "style" : style, "excess" : excess, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rounding_angled_edge_mask(OpenSCADObject):
    def __init__(self, h=None, r=None, r1=None, r2=None, d=None, d1=None, d2=None, ang=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rounding_angled_edge_mask", {"h" : h, "r" : r, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "ang" : ang, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rounding_angled_corner_mask(OpenSCADObject):
    def __init__(self, r=None, ang=None, d=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rounding_angled_corner_mask", {"r" : r, "ang" : ang, "d" : d, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rounding_cylinder_mask(OpenSCADObject):
    def __init__(self, r=None, rounding=None, d=None, **kwargs):
       super().__init__("rounding_cylinder_mask", {"r" : r, "rounding" : rounding, "d" : d, **kwargs})

class rounding_hole_mask(OpenSCADObject):
    def __init__(self, r=None, rounding=None, excess=None, d=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rounding_hole_mask", {"r" : r, "rounding" : rounding, "excess" : excess, "d" : d, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class teardrop_edge_mask(OpenSCADObject):
    def __init__(self, l=None, r=None, angle=None, excess=None, d=None, **kwargs):
       super().__init__("teardrop_edge_mask", {"l" : l, "r" : r, "angle" : angle, "excess" : excess, "d" : d, **kwargs})

class teardrop_corner_mask(OpenSCADObject):
    def __init__(self, r=None, angle=None, excess=None, d=None, **kwargs):
       super().__init__("teardrop_corner_mask", {"r" : r, "angle" : angle, "excess" : excess, "d" : d, **kwargs})

class chamfer_edge_mask(OpenSCADObject):
    def __init__(self, l=None, chamfer=None, excess=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("chamfer_edge_mask", {"l" : l, "chamfer" : chamfer, "excess" : excess, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class chamfer_corner_mask(OpenSCADObject):
    def __init__(self, chamfer=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("chamfer_corner_mask", {"chamfer" : chamfer, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class chamfer_cylinder_mask(OpenSCADObject):
    def __init__(self, r=None, chamfer=None, d=None, ang=None, from_end=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("chamfer_cylinder_mask", {"r" : r, "chamfer" : chamfer, "d" : d, "ang" : ang, "from_end" : from_end, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rounding_edge_mask(OpenSCADObject):
    def __init__(self, l=None, r=None, r1=None, r2=None, d=None, d1=None, d2=None, excess=None, anchor=None, spin=None, orient=None, h=None, **kwargs):
       super().__init__("rounding_edge_mask", {"l" : l, "r" : r, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "excess" : excess, "anchor" : anchor, "spin" : spin, "orient" : orient, "h" : h, **kwargs})

class rounding_corner_mask(OpenSCADObject):
    def __init__(self, r=None, d=None, style=None, excess=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rounding_corner_mask", {"r" : r, "d" : d, "style" : style, "excess" : excess, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rounding_angled_edge_mask(OpenSCADObject):
    def __init__(self, h=None, r=None, r1=None, r2=None, d=None, d1=None, d2=None, ang=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rounding_angled_edge_mask", {"h" : h, "r" : r, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "ang" : ang, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rounding_angled_corner_mask(OpenSCADObject):
    def __init__(self, r=None, ang=None, d=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rounding_angled_corner_mask", {"r" : r, "ang" : ang, "d" : d, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rounding_cylinder_mask(OpenSCADObject):
    def __init__(self, r=None, rounding=None, d=None, **kwargs):
       super().__init__("rounding_cylinder_mask", {"r" : r, "rounding" : rounding, "d" : d, **kwargs})

class rounding_hole_mask(OpenSCADObject):
    def __init__(self, r=None, rounding=None, excess=None, d=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rounding_hole_mask", {"r" : r, "rounding" : rounding, "excess" : excess, "d" : d, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class teardrop_edge_mask(OpenSCADObject):
    def __init__(self, l=None, r=None, angle=None, excess=None, d=None, **kwargs):
       super().__init__("teardrop_edge_mask", {"l" : l, "r" : r, "angle" : angle, "excess" : excess, "d" : d, **kwargs})

class teardrop_corner_mask(OpenSCADObject):
    def __init__(self, r=None, angle=None, excess=None, d=None, **kwargs):
       super().__init__("teardrop_corner_mask", {"r" : r, "angle" : angle, "excess" : excess, "d" : d, **kwargs})




#! /usr/bin/env python

from solid2 import *

rotation_speed = CustomizerSliderVariable("rotation_speed", 1.0)

set_global_viewport_translation([4, 3, 15])
set_global_viewport_rotation([60, 0, rotation_speed * get_animation_time() * 360])
set_global_viewport_distance(100)

c = cube(get_animation_time() * 10)

scad_render_to_file(c)




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/drawing.scad'}", use_not_include=False)

class stroke(OpenSCADObject):
    def __init__(self, path=None, width=None, closed=None, endcaps=None, endcap1=None, endcap2=None, joints=None, dots=None, endcap_width=None, endcap_width1=None, endcap_width2=None, joint_width=None, dots_width=None, endcap_length=None, endcap_length1=None, endcap_length2=None, joint_length=None, dots_length=None, endcap_extent=None, endcap_extent1=None, endcap_extent2=None, joint_extent=None, dots_extent=None, endcap_angle=None, endcap_angle1=None, endcap_angle2=None, joint_angle=None, dots_angle=None, endcap_color=None, endcap_color1=None, endcap_color2=None, joint_color=None, dots_color=None, color=None, trim=None, trim1=None, trim2=None, convexity=None, hull=None, **kwargs):
       super().__init__("stroke", {"path" : path, "width" : width, "closed" : closed, "endcaps" : endcaps, "endcap1" : endcap1, "endcap2" : endcap2, "joints" : joints, "dots" : dots, "endcap_width" : endcap_width, "endcap_width1" : endcap_width1, "endcap_width2" : endcap_width2, "joint_width" : joint_width, "dots_width" : dots_width, "endcap_length" : endcap_length, "endcap_length1" : endcap_length1, "endcap_length2" : endcap_length2, "joint_length" : joint_length, "dots_length" : dots_length, "endcap_extent" : endcap_extent, "endcap_extent1" : endcap_extent1, "endcap_extent2" : endcap_extent2, "joint_extent" : joint_extent, "dots_extent" : dots_extent, "endcap_angle" : endcap_angle, "endcap_angle1" : endcap_angle1, "endcap_angle2" : endcap_angle2, "joint_angle" : joint_angle, "dots_angle" : dots_angle, "endcap_color" : endcap_color, "endcap_color1" : endcap_color1, "endcap_color2" : endcap_color2, "joint_color" : joint_color, "dots_color" : dots_color, "color" : color, "trim" : trim, "trim1" : trim1, "trim2" : trim2, "convexity" : convexity, "hull" : hull, **kwargs})

class dashed_stroke(OpenSCADObject):
    def __init__(self, path=None, dashpat=None, width=None, closed=None, **kwargs):
       super().__init__("dashed_stroke", {"path" : path, "dashpat" : dashpat, "width" : width, "closed" : closed, **kwargs})

class arc(OpenSCADObject):
    def __init__(self, n=None, r=None, angle=None, d=None, cp=None, points=None, corner=None, width=None, thickness=None, start=None, wedge=None, anchor=None, spin=None, **kwargs):
       super().__init__("arc", {"n" : n, "r" : r, "angle" : angle, "d" : d, "cp" : cp, "points" : points, "corner" : corner, "width" : width, "thickness" : thickness, "start" : start, "wedge" : wedge, "anchor" : anchor, "spin" : spin, **kwargs})

class helix(OpenSCADObject):
    def __init__(self, l=None, h=None, turns=None, angle=None, r=None, r1=None, r2=None, d=None, d1=None, d2=None, **kwargs):
       super().__init__("helix", {"l" : l, "h" : h, "turns" : turns, "angle" : angle, "r" : r, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, **kwargs})

class turtle(OpenSCADObject):
    def __init__(self, commands=None, state=None, full_state=None, repeat=None, **kwargs):
       super().__init__("turtle", {"commands" : commands, "state" : state, "full_state" : full_state, "repeat" : repeat, **kwargs})

class debug_polygon(OpenSCADObject):
    def __init__(self, points=None, paths=None, vertices=None, edges=None, convexity=None, size=None, **kwargs):
       super().__init__("debug_polygon", {"points" : points, "paths" : paths, "vertices" : vertices, "edges" : edges, "convexity" : convexity, "size" : size, **kwargs})

class stroke(OpenSCADObject):
    def __init__(self, path=None, width=None, closed=None, endcaps=None, endcap1=None, endcap2=None, joints=None, dots=None, endcap_width=None, endcap_width1=None, endcap_width2=None, joint_width=None, dots_width=None, endcap_length=None, endcap_length1=None, endcap_length2=None, joint_length=None, dots_length=None, endcap_extent=None, endcap_extent1=None, endcap_extent2=None, joint_extent=None, dots_extent=None, endcap_angle=None, endcap_angle1=None, endcap_angle2=None, joint_angle=None, dots_angle=None, endcap_color=None, endcap_color1=None, endcap_color2=None, joint_color=None, dots_color=None, color=None, trim=None, trim1=None, trim2=None, convexity=None, hull=None, **kwargs):
       super().__init__("stroke", {"path" : path, "width" : width, "closed" : closed, "endcaps" : endcaps, "endcap1" : endcap1, "endcap2" : endcap2, "joints" : joints, "dots" : dots, "endcap_width" : endcap_width, "endcap_width1" : endcap_width1, "endcap_width2" : endcap_width2, "joint_width" : joint_width, "dots_width" : dots_width, "endcap_length" : endcap_length, "endcap_length1" : endcap_length1, "endcap_length2" : endcap_length2, "joint_length" : joint_length, "dots_length" : dots_length, "endcap_extent" : endcap_extent, "endcap_extent1" : endcap_extent1, "endcap_extent2" : endcap_extent2, "joint_extent" : joint_extent, "dots_extent" : dots_extent, "endcap_angle" : endcap_angle, "endcap_angle1" : endcap_angle1, "endcap_angle2" : endcap_angle2, "joint_angle" : joint_angle, "dots_angle" : dots_angle, "endcap_color" : endcap_color, "endcap_color1" : endcap_color1, "endcap_color2" : endcap_color2, "joint_color" : joint_color, "dots_color" : dots_color, "color" : color, "trim" : trim, "trim1" : trim1, "trim2" : trim2, "convexity" : convexity, "hull" : hull, **kwargs})

class dashed_stroke(OpenSCADObject):
    def __init__(self, path=None, dashpat=None, closed=None, **kwargs):
       super().__init__("dashed_stroke", {"path" : path, "dashpat" : dashpat, "closed" : closed, **kwargs})

class arc(OpenSCADObject):
    def __init__(self, n=None, r=None, angle=None, d=None, cp=None, points=None, corner=None, width=None, thickness=None, start=None, wedge=None, long=None, cw=None, ccw=None, endpoint=None, **kwargs):
       super().__init__("arc", {"n" : n, "r" : r, "angle" : angle, "d" : d, "cp" : cp, "points" : points, "corner" : corner, "width" : width, "thickness" : thickness, "start" : start, "wedge" : wedge, "long" : long, "cw" : cw, "ccw" : ccw, "endpoint" : endpoint, **kwargs})

class helix(OpenSCADObject):
    def __init__(self, l=None, h=None, turns=None, angle=None, r=None, r1=None, r2=None, d=None, d1=None, d2=None, **kwargs):
       super().__init__("helix", {"l" : l, "h" : h, "turns" : turns, "angle" : angle, "r" : r, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, **kwargs})

class _normal_segment(OpenSCADObject):
    def __init__(self, p1=None, p2=None, **kwargs):
       super().__init__("_normal_segment", {"p1" : p1, "p2" : p2, **kwargs})

class turtle(OpenSCADObject):
    def __init__(self, commands=None, state=None, full_state=None, repeat=None, **kwargs):
       super().__init__("turtle", {"commands" : commands, "state" : state, "full_state" : full_state, "repeat" : repeat, **kwargs})

class _turtle_repeat(OpenSCADObject):
    def __init__(self, commands=None, state=None, full_state=None, repeat=None, **kwargs):
       super().__init__("_turtle_repeat", {"commands" : commands, "state" : state, "full_state" : full_state, "repeat" : repeat, **kwargs})

class _turtle_command_len(OpenSCADObject):
    def __init__(self, commands=None, index=None, **kwargs):
       super().__init__("_turtle_command_len", {"commands" : commands, "index" : index, **kwargs})

class _turtle(OpenSCADObject):
    def __init__(self, commands=None, state=None, full_state=None, index=None, **kwargs):
       super().__init__("_turtle", {"commands" : commands, "state" : state, "full_state" : full_state, "index" : index, **kwargs})

class _turtle_command(OpenSCADObject):
    def __init__(self, command=None, parm=None, parm2=None, state=None, index=None, **kwargs):
       super().__init__("_turtle_command", {"command" : command, "parm" : parm, "parm2" : parm2, "state" : state, "index" : index, **kwargs})




#! /usr/bin/env python

import os
from pathlib import Path

p = Path(__file__).parent

for f in sorted(p.iterdir()):
    if f.suffix == ".py" and f.name[0].isdigit() \
                 and f.name[1].isdigit() and f.name[2] == '-':
        print(f)
        os.system(f'python3 {f.as_posix()}')



from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/regions.scad'}", use_not_include=False)

class region(OpenSCADObject):
    def __init__(self, r=None, anchor=None, spin=None, cp=None, atype=None, **kwargs):
       super().__init__("region", {"r" : r, "anchor" : anchor, "spin" : spin, "cp" : cp, "atype" : atype, **kwargs})

class exclusive_or(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("exclusive_or", {**kwargs})

class is_region(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("is_region", {"x" : x, **kwargs})

class is_valid_region(OpenSCADObject):
    def __init__(self, region=None, eps=None, **kwargs):
       super().__init__("is_valid_region", {"region" : region, "eps" : eps, **kwargs})

class _polygon_crosses_region(OpenSCADObject):
    def __init__(self, region=None, poly=None, eps=None, **kwargs):
       super().__init__("_polygon_crosses_region", {"region" : region, "poly" : poly, "eps" : eps, **kwargs})

class is_region_simple(OpenSCADObject):
    def __init__(self, region=None, eps=None, **kwargs):
       super().__init__("is_region_simple", {"region" : region, "eps" : eps, **kwargs})

class make_region(OpenSCADObject):
    def __init__(self, polys=None, nonzero=None, eps=None, **kwargs):
       super().__init__("make_region", {"polys" : polys, "nonzero" : nonzero, "eps" : eps, **kwargs})

class force_region(OpenSCADObject):
    def __init__(self, poly=None, **kwargs):
       super().__init__("force_region", {"poly" : poly, **kwargs})

class point_in_region(OpenSCADObject):
    def __init__(self, point=None, region=None, eps=None, **kwargs):
       super().__init__("point_in_region", {"point" : point, "region" : region, "eps" : eps, **kwargs})

class _point_in_region(OpenSCADObject):
    def __init__(self, point=None, region=None, eps=None, i=None, cnt=None, **kwargs):
       super().__init__("_point_in_region", {"point" : point, "region" : region, "eps" : eps, "i" : i, "cnt" : cnt, **kwargs})

class region_area(OpenSCADObject):
    def __init__(self, region=None, **kwargs):
       super().__init__("region_area", {"region" : region, **kwargs})

class _clockwise_region(OpenSCADObject):
    def __init__(self, r=None, **kwargs):
       super().__init__("_clockwise_region", {"r" : r, **kwargs})

class are_regions_equal(OpenSCADObject):
    def __init__(self, region1=None, region2=None, either_winding=None, **kwargs):
       super().__init__("are_regions_equal", {"region1" : region1, "region2" : region2, "either_winding" : either_winding, **kwargs})

class __are_regions_equal(OpenSCADObject):
    def __init__(self, region1=None, region2=None, i=None, **kwargs):
       super().__init__("__are_regions_equal", {"region1" : region1, "region2" : region2, "i" : i, **kwargs})

class _region_region_intersections(OpenSCADObject):
    def __init__(self, region1=None, region2=None, closed1=None, closed2=None, eps=None, **kwargs):
       super().__init__("_region_region_intersections", {"region1" : region1, "region2" : region2, "closed1" : closed1, "closed2" : closed2, "eps" : eps, **kwargs})

class split_region_at_region_crossings(OpenSCADObject):
    def __init__(self, region1=None, region2=None, closed1=None, closed2=None, eps=None, **kwargs):
       super().__init__("split_region_at_region_crossings", {"region1" : region1, "region2" : region2, "closed1" : closed1, "closed2" : closed2, "eps" : eps, **kwargs})

class region_parts(OpenSCADObject):
    def __init__(self, region=None, **kwargs):
       super().__init__("region_parts", {"region" : region, **kwargs})

class _offset_chamfer(OpenSCADObject):
    def __init__(self, center=None, points=None, delta=None, **kwargs):
       super().__init__("_offset_chamfer", {"center" : center, "points" : points, "delta" : delta, **kwargs})

class _shift_segment(OpenSCADObject):
    def __init__(self, segment=None, d=None, **kwargs):
       super().__init__("_shift_segment", {"segment" : segment, "d" : d, **kwargs})

class _segment_extension(OpenSCADObject):
    def __init__(self, s1=None, s2=None, **kwargs):
       super().__init__("_segment_extension", {"s1" : s1, "s2" : s2, **kwargs})

class _makefaces(OpenSCADObject):
    def __init__(self, direction=None, startind=None, good=None, pointcount=None, closed=None, **kwargs):
       super().__init__("_makefaces", {"direction" : direction, "startind" : startind, "good" : good, "pointcount" : pointcount, "closed" : closed, **kwargs})

class _makefaces_recurse(OpenSCADObject):
    def __init__(self, startind1=None, startind2=None, numfirst=None, numsecond=None, lenlist=None, closed=None, firstind=None, secondind=None, faces=None, **kwargs):
       super().__init__("_makefaces_recurse", {"startind1" : startind1, "startind2" : startind2, "numfirst" : numfirst, "numsecond" : numsecond, "lenlist" : lenlist, "closed" : closed, "firstind" : firstind, "secondind" : secondind, "faces" : faces, **kwargs})

class _good_segments(OpenSCADObject):
    def __init__(self, path=None, d=None, shiftsegs=None, closed=None, quality=None, **kwargs):
       super().__init__("_good_segments", {"path" : path, "d" : d, "shiftsegs" : shiftsegs, "closed" : closed, "quality" : quality, **kwargs})

class _segment_good(OpenSCADObject):
    def __init__(self, path=None, pathseg_unit=None, pathseg_len=None, d=None, seg=None, alpha=None, index=None, **kwargs):
       super().__init__("_segment_good", {"path" : path, "pathseg_unit" : pathseg_unit, "pathseg_len" : pathseg_len, "d" : d, "seg" : seg, "alpha" : alpha, "index" : index, **kwargs})

class _point_dist(OpenSCADObject):
    def __init__(self, path=None, pathseg_unit=None, pathseg_len=None, pt=None, **kwargs):
       super().__init__("_point_dist", {"path" : path, "pathseg_unit" : pathseg_unit, "pathseg_len" : pathseg_len, "pt" : pt, **kwargs})

class offset(OpenSCADObject):
    def __init__(self, path=None, r=None, delta=None, chamfer=None, closed=None, check_valid=None, quality=None, return_faces=None, firstface_index=None, flip_faces=None, same_length=None, **kwargs):
       super().__init__("offset", {"path" : path, "r" : r, "delta" : delta, "chamfer" : chamfer, "closed" : closed, "check_valid" : check_valid, "quality" : quality, "return_faces" : return_faces, "firstface_index" : firstface_index, "flip_faces" : flip_faces, "same_length" : same_length, **kwargs})

class _filter_region_parts(OpenSCADObject):
    def __init__(self, region1=None, region2=None, keep=None, eps=None, **kwargs):
       super().__init__("_filter_region_parts", {"region1" : region1, "region2" : region2, "keep" : keep, "eps" : eps, **kwargs})

class _list_three(OpenSCADObject):
    def __init__(self, a=None, b=None, c=None, **kwargs):
       super().__init__("_list_three", {"a" : a, "b" : b, "c" : c, **kwargs})

class union(OpenSCADObject):
    def __init__(self, regions=None, b=None, c=None, eps=None, **kwargs):
       super().__init__("union", {"regions" : regions, "b" : b, "c" : c, "eps" : eps, **kwargs})

class difference(OpenSCADObject):
    def __init__(self, regions=None, b=None, c=None, eps=None, **kwargs):
       super().__init__("difference", {"regions" : regions, "b" : b, "c" : c, "eps" : eps, **kwargs})

class intersection(OpenSCADObject):
    def __init__(self, regions=None, b=None, c=None, eps=None, **kwargs):
       super().__init__("intersection", {"regions" : regions, "b" : b, "c" : c, "eps" : eps, **kwargs})

class exclusive_or(OpenSCADObject):
    def __init__(self, regions=None, b=None, c=None, eps=None, **kwargs):
       super().__init__("exclusive_or", {"regions" : regions, "b" : b, "c" : c, "eps" : eps, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/affine.scad'}", use_not_include=False)

class affine2d_identity(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("affine2d_identity", {**kwargs})

class affine2d_translate(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("affine2d_translate", {"v" : v, **kwargs})

class affine2d_scale(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("affine2d_scale", {"v" : v, **kwargs})

class affine2d_zrot(OpenSCADObject):
    def __init__(self, ang=None, **kwargs):
       super().__init__("affine2d_zrot", {"ang" : ang, **kwargs})

class affine2d_mirror(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("affine2d_mirror", {"v" : v, **kwargs})

class affine2d_skew(OpenSCADObject):
    def __init__(self, xa=None, ya=None, **kwargs):
       super().__init__("affine2d_skew", {"xa" : xa, "ya" : ya, **kwargs})

class affine3d_identity(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("affine3d_identity", {**kwargs})

class affine3d_translate(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("affine3d_translate", {"v" : v, **kwargs})

class affine3d_scale(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("affine3d_scale", {"v" : v, **kwargs})

class affine3d_xrot(OpenSCADObject):
    def __init__(self, ang=None, **kwargs):
       super().__init__("affine3d_xrot", {"ang" : ang, **kwargs})

class affine3d_yrot(OpenSCADObject):
    def __init__(self, ang=None, **kwargs):
       super().__init__("affine3d_yrot", {"ang" : ang, **kwargs})

class affine3d_zrot(OpenSCADObject):
    def __init__(self, ang=None, **kwargs):
       super().__init__("affine3d_zrot", {"ang" : ang, **kwargs})

class affine3d_rot_by_axis(OpenSCADObject):
    def __init__(self, u=None, ang=None, **kwargs):
       super().__init__("affine3d_rot_by_axis", {"u" : u, "ang" : ang, **kwargs})

class affine3d_rot_from_to(OpenSCADObject):
    def __init__(self, _from=None, to=None, **kwargs):
       super().__init__("affine3d_rot_from_to", {"_from" : _from, "to" : to, **kwargs})

class affine3d_mirror(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("affine3d_mirror", {"v" : v, **kwargs})

class affine3d_skew(OpenSCADObject):
    def __init__(self, sxy=None, sxz=None, syx=None, syz=None, szx=None, szy=None, **kwargs):
       super().__init__("affine3d_skew", {"sxy" : sxy, "sxz" : sxz, "syx" : syx, "syz" : syz, "szx" : szx, "szy" : szy, **kwargs})

class affine3d_skew_xy(OpenSCADObject):
    def __init__(self, xa=None, ya=None, **kwargs):
       super().__init__("affine3d_skew_xy", {"xa" : xa, "ya" : ya, **kwargs})

class affine3d_skew_xz(OpenSCADObject):
    def __init__(self, xa=None, za=None, **kwargs):
       super().__init__("affine3d_skew_xz", {"xa" : xa, "za" : za, **kwargs})

class affine3d_skew_yz(OpenSCADObject):
    def __init__(self, ya=None, za=None, **kwargs):
       super().__init__("affine3d_skew_yz", {"ya" : ya, "za" : za, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/masks2d.scad'}", use_not_include=False)

class mask2d_roundover(OpenSCADObject):
    def __init__(self, r=None, inset=None, excess=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_roundover", {"r" : r, "inset" : inset, "excess" : excess, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_cove(OpenSCADObject):
    def __init__(self, r=None, inset=None, excess=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_cove", {"r" : r, "inset" : inset, "excess" : excess, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_chamfer(OpenSCADObject):
    def __init__(self, edge=None, angle=None, inset=None, excess=None, x=None, y=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_chamfer", {"edge" : edge, "angle" : angle, "inset" : inset, "excess" : excess, "x" : x, "y" : y, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_rabbet(OpenSCADObject):
    def __init__(self, size=None, excess=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_rabbet", {"size" : size, "excess" : excess, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_dovetail(OpenSCADObject):
    def __init__(self, edge=None, angle=None, inset=None, shelf=None, excess=None, x=None, y=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_dovetail", {"edge" : edge, "angle" : angle, "inset" : inset, "shelf" : shelf, "excess" : excess, "x" : x, "y" : y, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_teardrop(OpenSCADObject):
    def __init__(self, r=None, angle=None, excess=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_teardrop", {"r" : r, "angle" : angle, "excess" : excess, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_ogee(OpenSCADObject):
    def __init__(self, pattern=None, excess=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_ogee", {"pattern" : pattern, "excess" : excess, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_roundover(OpenSCADObject):
    def __init__(self, r=None, inset=None, excess=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_roundover", {"r" : r, "inset" : inset, "excess" : excess, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_cove(OpenSCADObject):
    def __init__(self, r=None, inset=None, excess=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_cove", {"r" : r, "inset" : inset, "excess" : excess, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_chamfer(OpenSCADObject):
    def __init__(self, edge=None, angle=None, inset=None, excess=None, x=None, y=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_chamfer", {"edge" : edge, "angle" : angle, "inset" : inset, "excess" : excess, "x" : x, "y" : y, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_rabbet(OpenSCADObject):
    def __init__(self, size=None, excess=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_rabbet", {"size" : size, "excess" : excess, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_dovetail(OpenSCADObject):
    def __init__(self, edge=None, angle=None, inset=None, shelf=None, excess=None, x=None, y=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_dovetail", {"edge" : edge, "angle" : angle, "inset" : inset, "shelf" : shelf, "excess" : excess, "x" : x, "y" : y, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_teardrop(OpenSCADObject):
    def __init__(self, r=None, angle=None, excess=None, d=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_teardrop", {"r" : r, "angle" : angle, "excess" : excess, "d" : d, "anchor" : anchor, "spin" : spin, **kwargs})

class mask2d_ogee(OpenSCADObject):
    def __init__(self, pattern=None, excess=None, anchor=None, spin=None, **kwargs):
       super().__init__("mask2d_ogee", {"pattern" : pattern, "excess" : excess, "anchor" : anchor, "spin" : spin, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/fnliterals.scad'}", use_not_include=False)

class map(OpenSCADObject):
    def __init__(self, func=None, list=None, **kwargs):
       super().__init__("map", {"func" : func, "list" : list, **kwargs})

class filter(OpenSCADObject):
    def __init__(self, func=None, list=None, **kwargs):
       super().__init__("filter", {"func" : func, "list" : list, **kwargs})

class reduce(OpenSCADObject):
    def __init__(self, func=None, list=None, init=None, **kwargs):
       super().__init__("reduce", {"func" : func, "list" : list, "init" : init, **kwargs})

class accumulate(OpenSCADObject):
    def __init__(self, func=None, list=None, init=None, **kwargs):
       super().__init__("accumulate", {"func" : func, "list" : list, "init" : init, **kwargs})

class _while(OpenSCADObject):
    def __init__(self, init=None, cond=None, func=None, **kwargs):
       super().__init__("_while", {"init" : init, "cond" : cond, "func" : func, **kwargs})

class for_n(OpenSCADObject):
    def __init__(self, n=None, init=None, func=None, **kwargs):
       super().__init__("for_n", {"n" : n, "init" : init, "func" : func, **kwargs})

class find_all(OpenSCADObject):
    def __init__(self, func=None, list=None, **kwargs):
       super().__init__("find_all", {"func" : func, "list" : list, **kwargs})

class find_first(OpenSCADObject):
    def __init__(self, func=None, list=None, start=None, **kwargs):
       super().__init__("find_first", {"func" : func, "list" : list, "start" : start, **kwargs})

class binsearch(OpenSCADObject):
    def __init__(self, key=None, list=None, idx=None, cmp=None, **kwargs):
       super().__init__("binsearch", {"key" : key, "list" : list, "idx" : idx, "cmp" : cmp, **kwargs})

class simple_hash(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("simple_hash", {"x" : x, **kwargs})

class hashmap(OpenSCADObject):
    def __init__(self, hashsize=None, items=None, table=None, **kwargs):
       super().__init__("hashmap", {"hashsize" : hashsize, "items" : items, "table" : table, **kwargs})

class f_1arg(OpenSCADObject):
    def __init__(self, target_func=None, **kwargs):
       super().__init__("f_1arg", {"target_func" : target_func, **kwargs})

class f_2arg(OpenSCADObject):
    def __init__(self, target_func=None, **kwargs):
       super().__init__("f_2arg", {"target_func" : target_func, **kwargs})

class f_2arg_simple(OpenSCADObject):
    def __init__(self, target_func=None, **kwargs):
       super().__init__("f_2arg_simple", {"target_func" : target_func, **kwargs})

class f_3arg(OpenSCADObject):
    def __init__(self, target_func=None, **kwargs):
       super().__init__("f_3arg", {"target_func" : target_func, **kwargs})

class ival(OpenSCADObject):
    def __init__(self, target_func=None, **kwargs):
       super().__init__("ival", {"target_func" : target_func, **kwargs})

class xval(OpenSCADObject):
    def __init__(self, target_func=None, **kwargs):
       super().__init__("xval", {"target_func" : target_func, **kwargs})

class f_cmp(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_cmp", {"a" : a, "b" : b, **kwargs})

class f_gt(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_gt", {"a" : a, "b" : b, **kwargs})

class f_lt(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_lt", {"a" : a, "b" : b, **kwargs})

class f_gte(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_gte", {"a" : a, "b" : b, **kwargs})

class f_lte(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_lte", {"a" : a, "b" : b, **kwargs})

class f_eq(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_eq", {"a" : a, "b" : b, **kwargs})

class f_neq(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_neq", {"a" : a, "b" : b, **kwargs})

class f_approx(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_approx", {"a" : a, "b" : b, **kwargs})

class f_napprox(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_napprox", {"a" : a, "b" : b, **kwargs})

class f_or(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_or", {"a" : a, "b" : b, **kwargs})

class f_and(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_and", {"a" : a, "b" : b, **kwargs})

class f_nor(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_nor", {"a" : a, "b" : b, **kwargs})

class f_nand(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_nand", {"a" : a, "b" : b, **kwargs})

class f_xor(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_xor", {"a" : a, "b" : b, **kwargs})

class f_not(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_not", {"a" : a, **kwargs})

class f_even(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_even", {"a" : a, **kwargs})

class f_odd(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_odd", {"a" : a, **kwargs})

class f_add(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_add", {"a" : a, "b" : b, **kwargs})

class f_sub(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_sub", {"a" : a, "b" : b, **kwargs})

class f_mul(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_mul", {"a" : a, "b" : b, **kwargs})

class f_div(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_div", {"a" : a, "b" : b, **kwargs})

class f_mod(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_mod", {"a" : a, "b" : b, **kwargs})

class f_pow(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_pow", {"a" : a, "b" : b, **kwargs})

class f_neg(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_neg", {"a" : a, **kwargs})

class f_min(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_min", {"a" : a, **kwargs})

class f_max(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_max", {"a" : a, **kwargs})

class f_min2(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_min2", {"a" : a, "b" : b, **kwargs})

class f_max2(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_max2", {"a" : a, "b" : b, **kwargs})

class f_min3(OpenSCADObject):
    def __init__(self, a=None, b=None, c=None, **kwargs):
       super().__init__("f_min3", {"a" : a, "b" : b, "c" : c, **kwargs})

class f_max3(OpenSCADObject):
    def __init__(self, a=None, b=None, c=None, **kwargs):
       super().__init__("f_max3", {"a" : a, "b" : b, "c" : c, **kwargs})

class f_sin(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_sin", {"a" : a, **kwargs})

class f_cos(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_cos", {"a" : a, **kwargs})

class f_tan(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_tan", {"a" : a, **kwargs})

class f_asin(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_asin", {"a" : a, **kwargs})

class f_acos(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_acos", {"a" : a, **kwargs})

class f_atan(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_atan", {"a" : a, **kwargs})

class f_atan2(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_atan2", {"a" : a, "b" : b, **kwargs})

class f_len(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_len", {"a" : a, **kwargs})

class f_chr(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_chr", {"a" : a, **kwargs})

class f_ord(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_ord", {"a" : a, **kwargs})

class f_str(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_str", {"a" : a, **kwargs})

class f_str2(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_str2", {"a" : a, "b" : b, **kwargs})

class f_str3(OpenSCADObject):
    def __init__(self, a=None, b=None, c=None, **kwargs):
       super().__init__("f_str3", {"a" : a, "b" : b, "c" : c, **kwargs})

class f_floor(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_floor", {"a" : a, **kwargs})

class f_round(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_round", {"a" : a, **kwargs})

class f_ceil(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_ceil", {"a" : a, **kwargs})

class f_abs(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_abs", {"a" : a, **kwargs})

class f_sign(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_sign", {"a" : a, **kwargs})

class f_ln(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_ln", {"a" : a, **kwargs})

class f_log(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_log", {"a" : a, **kwargs})

class f_exp(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_exp", {"a" : a, **kwargs})

class f_sqr(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_sqr", {"a" : a, **kwargs})

class f_sqrt(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_sqrt", {"a" : a, **kwargs})

class f_norm(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_norm", {"a" : a, **kwargs})

class f_cross(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_cross", {"a" : a, "b" : b, **kwargs})

class f_is_def(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_def", {"x" : x, **kwargs})

class f_is_undef(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_undef", {"x" : x, **kwargs})

class f_is_bool(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_bool", {"x" : x, **kwargs})

class f_is_num(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_num", {"x" : x, **kwargs})

class f_is_int(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_int", {"x" : x, **kwargs})

class f_is_nan(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_nan", {"x" : x, **kwargs})

class f_is_finite(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_finite", {"x" : x, **kwargs})

class f_is_string(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_string", {"x" : x, **kwargs})

class f_is_list(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_list", {"x" : x, **kwargs})

class f_is_range(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_range", {"x" : x, **kwargs})

class f_is_function(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("f_is_function", {"x" : x, **kwargs})

class f_is_vector(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_is_vector", {"a" : a, "b" : b, **kwargs})

class f_is_path(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("f_is_path", {"a" : a, "b" : b, **kwargs})

class f_is_region(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_is_region", {"a" : a, **kwargs})

class f_is_vnf(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_is_vnf", {"a" : a, **kwargs})

class f_is_patch(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("f_is_patch", {"a" : a, **kwargs})




#! /usr/bin/env python

from solid2 import *

#set steps in OpenSCAD to 1000

ball_radius = 100

class Vector3:
    def __init__(self, x=0, y=0, z=0):
        self.x = x
        self.y = y
        self.z = z

    def __add__(self, other):
        return Vector3(self.x + other.x,
                       self.y + other.y,
                       self.z + other.z)

    def __mul__(self, factor):
        return Vector3(self.x * factor,
                       self.y * factor,
                       self.z * factor)

    def tolist(self):
        return [self.x, self.y, self.z]

#this is our little physics engine
def get_bouncing_ball_data(pos=Vector3(), vel=Vector3(5.0, 5.0, 200.0)):
    data = []
    gravity = Vector3(0.0, 0.0, -8.0)
    for t in range(1000):
        vel = (vel + gravity) * 0.995
        pos += vel
        if pos.z < ball_radius:
            vel.z = vel.z * -1
            pos.z = ball_radius

        data.append(pos.tolist())

    return data

set_global_viewport_translation([700, 900, 200])
set_global_viewport_rotation([80, 0, 20])
set_global_viewport_distance(6000)

#store pre computed data in global OpenSCAD variable
set_global_variable("bouncing_ball_data", get_bouncing_ball_data())

#get "dynamic ball position over time" from the data set
ball_pos_over_time = scad_inline("bouncing_ball_data[$t * 1000]")

#do some regular solid stuff with it
ball = translate(ball_pos_over_time)(sphere(ball_radius))
floor = background(cube([2000, 2000, 0.01]))

#render it with the pre computed data stored in the header
scad_render_to_file(ball + floor)

# I think if you really want to do something like this you could put the data
# into a seprate .scad file and include it into the main .scad file. You should
# also be able inject OpenSCAD functions (or modules) and use them...
#
# No clue what this could be good for, but....... I guess it might be possible
# to plug a -- more complex :-p -- physics engine into it and render animated
# movies....
#
# ....and print them with a 4d printer....?!?




#!env python3

import re
import os
import sys
import os.path
import argparse

from PIL import Image


def img2scad(filename, varname, resize, outf):
    indent = " " * 4
    im = Image.open(filename).convert('L')
    if resize:
        print("Resizing to {}x{}".format(resize[0],resize[1]))
        im = im.resize(resize)
    pix = im.load()
    width, height = im.size
    print("// Image {} ({}x{})".format(filename, width, height), file=outf)
    print("{} = [".format(varname), file=outf)
    line = indent
    for x in range(width):
        line += "[ "
        for y in range(height):
            line += "{:d}, ".format(pix[x,y])
            if len(line) > 60:
                print(line, file=outf)
                line = indent * 2
        line += " ],"
        if line != indent:
            print(line, file=outf)
            line = indent
    print("];", file=outf)
    print("", file=outf)


def main():
    parser = argparse.ArgumentParser(prog='img2scad')
    parser.add_argument('-o', '--outfile',
            help='Output .scad file.')
    parser.add_argument('-v', '--varname',
            help='Variable to use in .scad file.')
    parser.add_argument('-r', '--resize',
            help='Resample image to WIDTHxHEIGHT.')
    parser.add_argument('infile', help='Input image file.')
    opts = parser.parse_args()

    non_alnum = re.compile(r'[^a-zA-Z0-9_]')
    if not opts.varname:
        if opts.outfile:
            opts.varname = os.path.splitext(os.path.basename(opts.outfile))[0]
            opts.varname = non_alnum.sub("", opts.varname)
        else:
            opts.varname = "image_data"
    size_pat = re.compile(r'^([0-9][0-9]*)x([0-9][0-9]*)$')
    if opts.resize:
        m = size_pat.match(opts.resize)
        if not m:
            print("Expected WIDTHxHEIGHT resize format.", file=sys.stderr)
            sys.exit(-1)
        opts.resize = (int(m.group(1)), int(m.group(2)))

    if not opts.varname or non_alnum.search(opts.varname):
        print("Bad variable name: {}".format(opts.varname), file=sys.stderr)
        sys.exit(-1)

    if opts.outfile:
        with open(opts.outfile, "w") as outf:
            img2scad(opts.infile, opts.varname, opts.resize, outf)
    else:
        img2scad(opts.infile, opts.varname, opts.resize, sys.stdout)

    sys.exit(0)


if __name__ == "__main__":
    main()




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/cubetruss.scad'}", use_not_include=False)

_cubetruss_size = OpenSCADConstant('_cubetruss_size')
_cubetruss_strut_size = OpenSCADConstant('_cubetruss_strut_size')
_cubetruss_bracing = OpenSCADConstant('_cubetruss_bracing')
_cubetruss_clip_thickness = OpenSCADConstant('_cubetruss_clip_thickness')
class cubetruss_segment(OpenSCADObject):
    def __init__(self, size=None, strut=None, bracing=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cubetruss_segment", {"size" : size, "strut" : strut, "bracing" : bracing, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cubetruss_support(OpenSCADObject):
    def __init__(self, size=None, strut=None, extents=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cubetruss_support", {"size" : size, "strut" : strut, "extents" : extents, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cubetruss_clip(OpenSCADObject):
    def __init__(self, extents=None, size=None, strut=None, clipthick=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cubetruss_clip", {"extents" : extents, "size" : size, "strut" : strut, "clipthick" : clipthick, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cubetruss_foot(OpenSCADObject):
    def __init__(self, w=None, size=None, strut=None, clipthick=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cubetruss_foot", {"w" : w, "size" : size, "strut" : strut, "clipthick" : clipthick, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cubetruss_joiner(OpenSCADObject):
    def __init__(self, w=None, vert=None, size=None, strut=None, clipthick=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cubetruss_joiner", {"w" : w, "vert" : vert, "size" : size, "strut" : strut, "clipthick" : clipthick, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cubetruss_uclip(OpenSCADObject):
    def __init__(self, dual=None, size=None, strut=None, clipthick=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cubetruss_uclip", {"dual" : dual, "size" : size, "strut" : strut, "clipthick" : clipthick, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cubetruss(OpenSCADObject):
    def __init__(self, extents=None, clips=None, bracing=None, size=None, strut=None, clipthick=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cubetruss", {"extents" : extents, "clips" : clips, "bracing" : bracing, "size" : size, "strut" : strut, "clipthick" : clipthick, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cubetruss_corner(OpenSCADObject):
    def __init__(self, h=None, extents=None, bracing=None, size=None, strut=None, clipthick=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cubetruss_corner", {"h" : h, "extents" : extents, "bracing" : bracing, "size" : size, "strut" : strut, "clipthick" : clipthick, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cubetruss_dist(OpenSCADObject):
    def __init__(self, cubes=None, gaps=None, size=None, strut=None, **kwargs):
       super().__init__("cubetruss_dist", {"cubes" : cubes, "gaps" : gaps, "size" : size, "strut" : strut, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/nema_steppers.scad'}", use_not_include=False)

class nema11_stepper(OpenSCADObject):
    def __init__(self, h=None, shaft=None, shaft_len=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema11_stepper", {"h" : h, "shaft" : shaft, "shaft_len" : shaft_len, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema14_stepper(OpenSCADObject):
    def __init__(self, h=None, shaft=None, shaft_len=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema14_stepper", {"h" : h, "shaft" : shaft, "shaft_len" : shaft_len, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema17_stepper(OpenSCADObject):
    def __init__(self, h=None, shaft=None, shaft_len=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema17_stepper", {"h" : h, "shaft" : shaft, "shaft_len" : shaft_len, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema23_stepper(OpenSCADObject):
    def __init__(self, h=None, shaft=None, shaft_len=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema23_stepper", {"h" : h, "shaft" : shaft, "shaft_len" : shaft_len, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema34_stepper(OpenSCADObject):
    def __init__(self, h=None, shaft=None, shaft_len=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema34_stepper", {"h" : h, "shaft" : shaft, "shaft_len" : shaft_len, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema_mount_holes(OpenSCADObject):
    def __init__(self, size=None, depth=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema_mount_holes", {"size" : size, "depth" : depth, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema11_mount_holes(OpenSCADObject):
    def __init__(self, depth=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema11_mount_holes", {"depth" : depth, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema14_mount_holes(OpenSCADObject):
    def __init__(self, depth=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema14_mount_holes", {"depth" : depth, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema17_mount_holes(OpenSCADObject):
    def __init__(self, depth=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema17_mount_holes", {"depth" : depth, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema23_mount_holes(OpenSCADObject):
    def __init__(self, depth=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema23_mount_holes", {"depth" : depth, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema34_mount_holes(OpenSCADObject):
    def __init__(self, depth=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nema34_mount_holes", {"depth" : depth, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class nema_motor_width(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("nema_motor_width", {"size" : size, **kwargs})

class nema_motor_plinth_height(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("nema_motor_plinth_height", {"size" : size, **kwargs})

class nema_motor_plinth_diam(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("nema_motor_plinth_diam", {"size" : size, **kwargs})

class nema_motor_screw_spacing(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("nema_motor_screw_spacing", {"size" : size, **kwargs})

class nema_motor_screw_size(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("nema_motor_screw_size", {"size" : size, **kwargs})

class nema_motor_screw_depth(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("nema_motor_screw_depth", {"size" : size, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/linalg.scad'}", use_not_include=False)

class echo_matrix(OpenSCADObject):
    def __init__(self, M=None, description=None, sig=None, sep=None, eps=None, **kwargs):
       super().__init__("echo_matrix", {"M" : M, "description" : description, "sig" : sig, "sep" : sep, "eps" : eps, **kwargs})

class is_matrix(OpenSCADObject):
    def __init__(self, A=None, m=None, n=None, square=None, **kwargs):
       super().__init__("is_matrix", {"A" : A, "m" : m, "n" : n, "square" : square, **kwargs})

class is_matrix_symmetric(OpenSCADObject):
    def __init__(self, A=None, eps=None, **kwargs):
       super().__init__("is_matrix_symmetric", {"A" : A, "eps" : eps, **kwargs})

class is_rotation(OpenSCADObject):
    def __init__(self, A=None, dim=None, centered=None, **kwargs):
       super().__init__("is_rotation", {"A" : A, "dim" : dim, "centered" : centered, **kwargs})

class echo_matrix(OpenSCADObject):
    def __init__(self, M=None, description=None, sig=None, sep=None, eps=None, **kwargs):
       super().__init__("echo_matrix", {"M" : M, "description" : description, "sig" : sig, "sep" : sep, "eps" : eps, **kwargs})

class column(OpenSCADObject):
    def __init__(self, M=None, i=None, **kwargs):
       super().__init__("column", {"M" : M, "i" : i, **kwargs})

class submatrix(OpenSCADObject):
    def __init__(self, M=None, idx1=None, idx2=None, **kwargs):
       super().__init__("submatrix", {"M" : M, "idx1" : idx1, "idx2" : idx2, **kwargs})

class ident(OpenSCADObject):
    def __init__(self, n=None, **kwargs):
       super().__init__("ident", {"n" : n, **kwargs})

class diagonal_matrix(OpenSCADObject):
    def __init__(self, diag=None, offdiag=None, **kwargs):
       super().__init__("diagonal_matrix", {"diag" : diag, "offdiag" : offdiag, **kwargs})

class transpose(OpenSCADObject):
    def __init__(self, M=None, reverse=None, **kwargs):
       super().__init__("transpose", {"M" : M, "reverse" : reverse, **kwargs})

class outer_product(OpenSCADObject):
    def __init__(self, u=None, v=None, **kwargs):
       super().__init__("outer_product", {"u" : u, "v" : v, **kwargs})

class submatrix_set(OpenSCADObject):
    def __init__(self, M=None, A=None, m=None, n=None, **kwargs):
       super().__init__("submatrix_set", {"M" : M, "A" : A, "m" : m, "n" : n, **kwargs})

class hstack(OpenSCADObject):
    def __init__(self, M1=None, M2=None, M3=None, **kwargs):
       super().__init__("hstack", {"M1" : M1, "M2" : M2, "M3" : M3, **kwargs})

class block_matrix(OpenSCADObject):
    def __init__(self, M=None, **kwargs):
       super().__init__("block_matrix", {"M" : M, **kwargs})

class linear_solve(OpenSCADObject):
    def __init__(self, A=None, b=None, pivot=None, **kwargs):
       super().__init__("linear_solve", {"A" : A, "b" : b, "pivot" : pivot, **kwargs})

class linear_solve3(OpenSCADObject):
    def __init__(self, A=None, b=None, **kwargs):
       super().__init__("linear_solve3", {"A" : A, "b" : b, **kwargs})

class matrix_inverse(OpenSCADObject):
    def __init__(self, A=None, **kwargs):
       super().__init__("matrix_inverse", {"A" : A, **kwargs})

class rot_inverse(OpenSCADObject):
    def __init__(self, T=None, **kwargs):
       super().__init__("rot_inverse", {"T" : T, **kwargs})

class null_space(OpenSCADObject):
    def __init__(self, A=None, eps=None, **kwargs):
       super().__init__("null_space", {"A" : A, "eps" : eps, **kwargs})

class qr_factor(OpenSCADObject):
    def __init__(self, A=None, pivot=None, **kwargs):
       super().__init__("qr_factor", {"A" : A, "pivot" : pivot, **kwargs})

class _qr_factor(OpenSCADObject):
    def __init__(self, A=None, Q=None, P=None, pivot=None, col=None, m=None, n=None, **kwargs):
       super().__init__("_qr_factor", {"A" : A, "Q" : Q, "P" : P, "pivot" : pivot, "col" : col, "m" : m, "n" : n, **kwargs})

class _swap_matrix(OpenSCADObject):
    def __init__(self, n=None, i=None, j=None, **kwargs):
       super().__init__("_swap_matrix", {"n" : n, "i" : i, "j" : j, **kwargs})

class back_substitute(OpenSCADObject):
    def __init__(self, R=None, b=None, transpose=None, **kwargs):
       super().__init__("back_substitute", {"R" : R, "b" : b, "transpose" : transpose, **kwargs})

class _back_substitute(OpenSCADObject):
    def __init__(self, R=None, b=None, x=None, **kwargs):
       super().__init__("_back_substitute", {"R" : R, "b" : b, "x" : x, **kwargs})

class cholesky(OpenSCADObject):
    def __init__(self, A=None, **kwargs):
       super().__init__("cholesky", {"A" : A, **kwargs})

class _cholesky(OpenSCADObject):
    def __init__(self, A=None, L=None, n=None, **kwargs):
       super().__init__("_cholesky", {"A" : A, "L" : L, "n" : n, **kwargs})

class det2(OpenSCADObject):
    def __init__(self, M=None, **kwargs):
       super().__init__("det2", {"M" : M, **kwargs})

class det3(OpenSCADObject):
    def __init__(self, M=None, **kwargs):
       super().__init__("det3", {"M" : M, **kwargs})

class det4(OpenSCADObject):
    def __init__(self, M=None, **kwargs):
       super().__init__("det4", {"M" : M, **kwargs})

class determinant(OpenSCADObject):
    def __init__(self, M=None, **kwargs):
       super().__init__("determinant", {"M" : M, **kwargs})

class norm_fro(OpenSCADObject):
    def __init__(self, A=None, **kwargs):
       super().__init__("norm_fro", {"A" : A, **kwargs})

class matrix_trace(OpenSCADObject):
    def __init__(self, M=None, **kwargs):
       super().__init__("matrix_trace", {"M" : M, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/wiring.scad'}", use_not_include=False)

class wire_bundle(OpenSCADObject):
    def __init__(self, path=None, wires=None, wirediam=None, rounding=None, wirenum=None, corner_steps=None, **kwargs):
       super().__init__("wire_bundle", {"path" : path, "wires" : wires, "wirediam" : wirediam, "rounding" : rounding, "wirenum" : wirenum, "corner_steps" : corner_steps, **kwargs})

class _hex_offset_ring(OpenSCADObject):
    def __init__(self, d=None, lev=None, **kwargs):
       super().__init__("_hex_offset_ring", {"d" : d, "lev" : lev, **kwargs})

class _hex_offsets(OpenSCADObject):
    def __init__(self, n=None, d=None, lev=None, arr=None, **kwargs):
       super().__init__("_hex_offsets", {"n" : n, "d" : d, "lev" : lev, "arr" : arr, **kwargs})




import hashlib
import pickle
from functools import partial
from pathlib import Path
from collections import namedtuple

from ..config import config
from .object_base import OpenSCADConstant
from .utils import escape_openscad_identifier
from .object_factory import create_openscad_wrapper_from_symbols

callables_entry = namedtuple("callables_entry", ["name", "args", "kwargs"])

# ===========
# = Parsing =
# ===========
def get_pickle_filename(filename):
    md5 = hashlib.md5()
    md5.update(filename.as_posix().encode('utf-8'))
    filename_hash = md5.hexdigest()

    with open(filename, mode='rb') as f:
        d = hashlib.md5()
        for buf in iter(partial(f.read, 128), b''):
            d.update(buf)
        file_content_hash = d.hexdigest()

    pickle_filename = Path(filename_hash).with_suffix("." + file_content_hash).name
    return config.pickle_cache_dir / pickle_filename, filename_hash

def check_pickle_cache(filename):
    if not config.enable_pickle_cache:
        return None, None, False

    pickle_filename, _ = get_pickle_filename(filename)

    #create pickle_cache_dir if neccessary
    if not config.pickle_cache_dir.exists():
        config.pickle_cache_dir.mkdir(parents=True)

    #if the file exists, load and return it
    if pickle_filename.exists():
        with pickle_filename.open("rb") as f:
            pickler = pickle.Unpickler(f)
            callables = pickler.load()
            global_vars = pickler.load()
            return callables, global_vars, True

    return None, None, False

def update_pickle_cache(filename, callables, global_vars):
    if not config.enable_pickle_cache:
        return

    pickle_filename, filename_hash = get_pickle_filename(filename)

    #delete old files which correspond to this (absolute) filename(_hash)
    for f in pickle_filename.parent.glob(filename_hash + "*"):
        Path.unlink(f)

    #write the callables and global_vars to disk
    with pickle_filename.open("wb") as f:
        pickler = pickle.Pickler(f)
        pickler.dump(callables)
        pickler.dump(global_vars)

def get_scad_file_as_dict(filename):
    assert(filename.is_absolute())

    #check whether we have a valid up to date pickled version of it
    callables, global_vars, cache_is_valid = check_pickle_cache(filename)
    if not cache_is_valid:
        #otherwise parse the file
        from ..libs.py_scadparser import scad_parser
        modules, functions, global_vars = scad_parser.parseFile(filename)

        callables = []
        for c in modules + functions:
            #args = [p.name for p in c.parameters if not p.optional]
            #kwargs = [p.name for p in c.parameters if p.optional]
            kwargs = [p.name for p in c.parameters]

            callables.append(callables_entry(c.name, [], kwargs))

        #write the read version to disk so we can probably use it the next time
        update_pickle_cache(filename, callables, global_vars)

    #create a wrapper for each callable and add it to the dict
    new_namespace_dict = {}
    for c in callables:
        wrapper = create_openscad_wrapper_from_symbols(c.name, [], c.kwargs)
        new_namespace_dict[escape_openscad_identifier(c.name)] = wrapper

    for c in global_vars:
        new_namespace_dict[escape_openscad_identifier(c.name)] = \
                                                        OpenSCADConstant(c.name)

    return new_namespace_dict




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/screws.scad'}", use_not_include=False)

class screw(OpenSCADObject):
    def __init__(self, name=None, head=None, drive=None, thread=None, drive_size=None, oversize=None, spec=None, length=None, l=None, shank=None, tolerance=None, details=None, anchor=None, anchor_head=None, spin=None, orient=None, **kwargs):
       super().__init__("screw", {"name" : name, "head" : head, "drive" : drive, "thread" : thread, "drive_size" : drive_size, "oversize" : oversize, "spec" : spec, "length" : length, "l" : l, "shank" : shank, "tolerance" : tolerance, "details" : details, "anchor" : anchor, "anchor_head" : anchor_head, "spin" : spin, "orient" : orient, **kwargs})

class _driver(OpenSCADObject):
    def __init__(self, spec=None, **kwargs):
       super().__init__("_driver", {"spec" : spec, **kwargs})

class _rod(OpenSCADObject):
    def __init__(self, spec=None, length=None, tolerance=None, orient=None, spin=None, anchor=None, **kwargs):
       super().__init__("_rod", {"spec" : spec, "length" : length, "tolerance" : tolerance, "orient" : orient, "spin" : spin, "anchor" : anchor, **kwargs})

class nut(OpenSCADObject):
    def __init__(self, name=None, diameter=None, thickness=None, thread=None, oversize=None, spec=None, tolerance=None, bevel=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nut", {"name" : name, "diameter" : diameter, "thickness" : thickness, "thread" : thread, "oversize" : oversize, "spec" : spec, "tolerance" : tolerance, "bevel" : bevel, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class screw_head(OpenSCADObject):
    def __init__(self, screw_info=None, details=None, **kwargs):
       super().__init__("screw_head", {"screw_info" : screw_info, "details" : details, **kwargs})

class screw(OpenSCADObject):
    def __init__(self, name=None, head=None, drive=None, thread=None, drive_size=None, oversize=None, spec=None, length=None, l=None, shank=None, tolerance=None, details=None, anchor=None, anchor_head=None, spin=None, orient=None, **kwargs):
       super().__init__("screw", {"name" : name, "head" : head, "drive" : drive, "thread" : thread, "drive_size" : drive_size, "oversize" : oversize, "spec" : spec, "length" : length, "l" : l, "shank" : shank, "tolerance" : tolerance, "details" : details, "anchor" : anchor, "anchor_head" : anchor_head, "spin" : spin, "orient" : orient, **kwargs})

class _ISO_thread_tolerance(OpenSCADObject):
    def __init__(self, diameter=None, pitch=None, internal=None, tolerance=None, **kwargs):
       super().__init__("_ISO_thread_tolerance", {"diameter" : diameter, "pitch" : pitch, "internal" : internal, "tolerance" : tolerance, **kwargs})

class _UTS_thread_tolerance(OpenSCADObject):
    def __init__(self, diam=None, pitch=None, internal=None, tolerance=None, **kwargs):
       super().__init__("_UTS_thread_tolerance", {"diam" : diam, "pitch" : pitch, "internal" : internal, "tolerance" : tolerance, **kwargs})

class _exact_thread_tolerance(OpenSCADObject):
    def __init__(self, d=None, P=None, **kwargs):
       super().__init__("_exact_thread_tolerance", {"d" : d, "P" : P, **kwargs})

class nut(OpenSCADObject):
    def __init__(self, name=None, diameter=None, thickness=None, thread=None, oversize=None, spec=None, tolerance=None, bevel=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("nut", {"name" : name, "diameter" : diameter, "thickness" : thickness, "thread" : thread, "oversize" : oversize, "spec" : spec, "tolerance" : tolerance, "bevel" : bevel, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class _parse_screw_name(OpenSCADObject):
    def __init__(self, name=None, **kwargs):
       super().__init__("_parse_screw_name", {"name" : name, **kwargs})

class _parse_drive(OpenSCADObject):
    def __init__(self, drive=None, drive_size=None, **kwargs):
       super().__init__("_parse_drive", {"drive" : drive, "drive_size" : drive_size, **kwargs})

class screw_head(OpenSCADObject):
    def __init__(self, screw_info=None, details=None, **kwargs):
       super().__init__("screw_head", {"screw_info" : screw_info, "details" : details, **kwargs})

class screw_info(OpenSCADObject):
    def __init__(self, name=None, head=None, drive=None, thread=None, drive_size=None, oversize=None, **kwargs):
       super().__init__("screw_info", {"name" : name, "head" : head, "drive" : drive, "thread" : thread, "drive_size" : drive_size, "oversize" : oversize, **kwargs})

class _screw_info_english(OpenSCADObject):
    def __init__(self, diam=None, threadcount=None, head=None, thread=None, drive=None, **kwargs):
       super().__init__("_screw_info_english", {"diam" : diam, "threadcount" : threadcount, "head" : head, "thread" : thread, "drive" : drive, **kwargs})

class _screw_info_metric(OpenSCADObject):
    def __init__(self, diam=None, pitch=None, head=None, thread=None, drive=None, **kwargs):
       super().__init__("_screw_info_metric", {"diam" : diam, "pitch" : pitch, "head" : head, "thread" : thread, "drive" : drive, **kwargs})

class _is_positive(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("_is_positive", {"x" : x, **kwargs})

class _validate_screw_spec(OpenSCADObject):
    def __init__(self, spec=None, **kwargs):
       super().__init__("_validate_screw_spec", {"spec" : spec, **kwargs})

class thread_specification(OpenSCADObject):
    def __init__(self, screw_spec=None, tolerance=None, internal=None, **kwargs):
       super().__init__("thread_specification", {"screw_spec" : screw_spec, "tolerance" : tolerance, "internal" : internal, **kwargs})




#! /usr/bin/env python

from solid2 import *

set_global_fn(32)
set_global_viewport_distance(abs(sin(get_animation_time() * 360)) * 10 + 5)
set_global_viewport_translation([0, -1, 0])
set_global_viewport_rotation([63, 0, get_animation_time() * 360])
set_global_viewport_fov(25)

def funny_cube():
    customized_color = CustomizerDropdownVariable(name = "cube_color",
                                                  default_value = "blue",
                                                  options = ["red", "green", "blue"],
                                                  label = "The color of the cube",
                                                  tab="Colors")

    customized_animation_factor = CustomizerSliderVariable(name = "anim_factor",
                                                           default_value = 1,
                                                           min_ = 1,
                                                           max_ = 10,
                                                           step = 0.5,
                                                           label = "Animation speed factor",
                                                           tab = "Animation")

    return color(customized_color) (
                cube(abs(sin(get_animation_time() * 360 * customized_animation_factor)), center=True)
           )

def funny_sphere():
    customized_color = ScadValue("cube_color")
    customized_animation_factor = ScadValue("anim_factor")

    return translate([0, -2, 0]) (
                color(customized_color) (
                    sphere(r = abs(sin(get_animation_time() * 360 * customized_animation_factor - 90)))
                )
           )

def do_nots():
    customized_color = ScadValue("cube_color")
    customized_animation_factor = ScadValue("anim_factor")

    #if customized_color == "blue":
    #    print("This causes a python runtime error!")

    #for i in range(customized_animation_factor):
    #    print("This causes a python runtime error!")

    #f = 1.0
    #f *= customized_animation_factor
    #for i in range(f):
    #    print("This causes a python runtime error! (and this is why it is called greedy)")

(funny_cube() + funny_sphere()).save_as_scad()



from solid2.extensions.bosl2 import *

def cubic_barbell(s=100, anchor=CENTER, spin=0, orient=UP):
    return attachable(anchor,spin,orient, size=[s*3,s,s]) (
        union() (
            xcopies(2*s)(cube(s, center=True)),
            xcyl(h=2*s, d=s/4)
        )
    )

#cc = cubic_barbell2(100) (
#        attach(TOP)(cube(50)),
#        attach(LEFT)(sphere(50))
#    )

cc = cubic_barbell(100)(
        cube(50, center=True).attach(TOP),
        sphere(50).attach(LEFT),
        show_anchors(30)
    )

cc.save_as_scad()




from .version import *
from .constants import *
from .transforms import *
from .distributors import *
from .mutators import *
from .color import *
from .attachments import *
from .shapes3d import *
from .shapes2d import *
from .drawing import *
from .masks3d import *
from .masks2d import *
from .math import *
from .paths import *
from .lists import *
from .comparisons import *
from .linalg import *
from .trigonometry import *
from .vectors import *
from .affine import *
from .coords import *
from .geometry import *
from .regions import *
from .strings import *
from .skin import *
from .vnf import *
from .utility import *
from .partitions import *
from . import wiring
from . import linear_bearings
from . import bottlecaps
from . import fnliterals
from . import beziers
from . import screws
from . import polyhedra
from . import gears
from . import rounding
from . import turtle3d
from . import structs
from . import hingesnaps
from . import threading
from . import tripod_mounts
from . import sliders
from . import screw_drive
from . import nema_steppers
from . import walls
from . import joiners
from . import cubetruss
from . import modular_hose
from . import metric_screws



#!/usr/bin/env python3

import os
import operator

funcs = {}
for filename in os.listdir("."):
    if filename.endswith(".scad"):
        filepath = os.path.join(".",filename)
        with open(filepath, "r") as f:
            for linenum,line in enumerate(f.readlines()):
                if line.startswith("function "):
                    funcname = line[9:].strip().split("(")[0].strip()
                    if funcname.startswith("_"):
                        continue
                    if funcname in funcs:
                        print("WARNING!!! Function {} re-defined at {}:{}".format(funcname, filename, linenum+1));
                        print("           Previously defined at {}:{}".format(*funcs[funcname]));
                    else:
                        funcs[funcname] = (filename, linenum+1)

covered = {}
uncovered = funcs.copy()
for filename in os.listdir("tests"):
    if filename.startswith("test_") and filename.endswith(".scad"):
        filepath = os.path.join("tests",filename)
        with open(filepath, "r") as f:
            for linenum,line in enumerate(f.readlines()):
                if line.startswith("module "):
                    testmodule = line[7:].strip().split("(")[0].strip()
                    if testmodule.startswith("test_"):
                        funcname = testmodule.split("_",1)[1]
                        if funcname in uncovered:
                            if filename != "test_" + uncovered[funcname][0]:
                                print("WARNING!!! Function {} defined at {}:{}".format(funcname, *uncovered[funcname]));
                                print("           but tested at {}:{}".format(filename, linenum+1));
                            covered[funcname] = (filename,linenum+1)
                            del uncovered[funcname]

uncovered_by_file = {}
for funcname in sorted(list(uncovered.keys())):
    filename = uncovered[funcname][0]
    if filename not in uncovered_by_file:
        uncovered_by_file[filename] = []
    uncovered_by_file[filename].append(funcname)

mostest = []
for filename in uncovered_by_file.keys():
    mostest.append( (len(uncovered_by_file[filename]), filename) )

# for funcname in sorted(covered):
#     print("COVERED: function {}".format(funcname))

print("NOT COVERED:")
for cnt, filename in sorted(mostest, key=operator.itemgetter(0)):
    filefuncs = uncovered_by_file[filename]
    print("  {}: {:d} uncovered functions".format(filename, cnt))
    for funcname in filefuncs:
        print("    {}".format(funcname))

totfuncs = len(funcs.keys())
covfuncs = len(covered)

print(
    "Total coverage: {} of {} functions ({:.2f}%)".format(
        covfuncs, totfuncs, 100.0*covfuncs/totfuncs
    )
)

# vim: expandtab tabstop=4 shiftwidth=4 softtabstop=4 nowrap



from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/turtle3d.scad'}", use_not_include=False)

class turtle3d(OpenSCADObject):
    def __init__(self, commands=None, state=None, transforms=None, full_state=None, repeat=None, **kwargs):
       super().__init__("turtle3d", {"commands" : commands, "state" : state, "transforms" : transforms, "full_state" : full_state, "repeat" : repeat, **kwargs})

class _transpart(OpenSCADObject):
    def __init__(self, T=None, **kwargs):
       super().__init__("_transpart", {"T" : T, **kwargs})

class _rotpart(OpenSCADObject):
    def __init__(self, T=None, **kwargs):
       super().__init__("_rotpart", {"T" : T, **kwargs})

class _turtle3d_state_valid(OpenSCADObject):
    def __init__(self, state=None, **kwargs):
       super().__init__("_turtle3d_state_valid", {"state" : state, **kwargs})

class turtle3d(OpenSCADObject):
    def __init__(self, commands=None, state=None, transforms=None, full_state=None, repeat=None, **kwargs):
       super().__init__("turtle3d", {"commands" : commands, "state" : state, "transforms" : transforms, "full_state" : full_state, "repeat" : repeat, **kwargs})

class _turtle3d_repeat(OpenSCADObject):
    def __init__(self, commands=None, state=None, repeat=None, **kwargs):
       super().__init__("_turtle3d_repeat", {"commands" : commands, "state" : state, "repeat" : repeat, **kwargs})

class _turtle3d_command_len(OpenSCADObject):
    def __init__(self, commands=None, index=None, **kwargs):
       super().__init__("_turtle3d_command_len", {"commands" : commands, "index" : index, **kwargs})

class _turtle3d(OpenSCADObject):
    def __init__(self, commands=None, state=None, index=None, **kwargs):
       super().__init__("_turtle3d", {"commands" : commands, "state" : state, "index" : index, **kwargs})

class _turtle3d_rotation(OpenSCADObject):
    def __init__(self, command=None, angle=None, center=None, **kwargs):
       super().__init__("_turtle3d_rotation", {"command" : command, "angle" : angle, "center" : center, **kwargs})

class _tupdate(OpenSCADObject):
    def __init__(self, state=None, tran=None, pretran=None, **kwargs):
       super().__init__("_tupdate", {"state" : state, "tran" : tran, "pretran" : pretran, **kwargs})

class _turtle3d_command(OpenSCADObject):
    def __init__(self, command=None, parm=None, parm2=None, state=None, index=None, **kwargs):
       super().__init__("_turtle3d_command", {"command" : command, "parm" : parm, "parm2" : parm2, "state" : state, "index" : index, **kwargs})

class _turtle3d_list_command(OpenSCADObject):
    def __init__(self, command=None, arcsteps=None, movescale=None, lastT=None, lastPre=None, index=None, **kwargs):
       super().__init__("_turtle3d_list_command", {"command" : command, "arcsteps" : arcsteps, "movescale" : movescale, "lastT" : lastT, "lastPre" : lastPre, "index" : index, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/walls.scad'}", use_not_include=False)

class sparse_wall(OpenSCADObject):
    def __init__(self, h=None, l=None, thick=None, maxang=None, strut=None, max_bridge=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("sparse_wall", {"h" : h, "l" : l, "thick" : thick, "maxang" : maxang, "strut" : strut, "max_bridge" : max_bridge, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class corrugated_wall(OpenSCADObject):
    def __init__(self, h=None, l=None, thick=None, strut=None, wall=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("corrugated_wall", {"h" : h, "l" : l, "thick" : thick, "strut" : strut, "wall" : wall, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class thinning_wall(OpenSCADObject):
    def __init__(self, h=None, l=None, thick=None, ang=None, braces=None, strut=None, wall=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("thinning_wall", {"h" : h, "l" : l, "thick" : thick, "ang" : ang, "braces" : braces, "strut" : strut, "wall" : wall, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class thinning_triangle(OpenSCADObject):
    def __init__(self, h=None, l=None, thick=None, ang=None, strut=None, wall=None, diagonly=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("thinning_triangle", {"h" : h, "l" : l, "thick" : thick, "ang" : ang, "strut" : strut, "wall" : wall, "diagonly" : diagonly, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class narrowing_strut(OpenSCADObject):
    def __init__(self, w=None, l=None, wall=None, ang=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("narrowing_strut", {"w" : w, "l" : l, "wall" : wall, "ang" : ang, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})




#only import the external interface!
from .builtins import *

from .scad_render import scad_render,\
                         scad_render_to_file,\
                         render_to_stl_file

from .scad_import import import_scad, use, include

from .object_base import scad_inline, scad_inline_parameter_func, sin, cos,\
                         tan, asin, acos, atan, sqrt, not_



#! /usr/bin/env python

from solid2 import *

# you can store any "openscad" object in a regular variable. No matter whether
# it's just a primitive or a complete assembly
c = cube([10, 20, 30])
s = sphere(10)

d = c - s
# Operators: This is from the operators extension.
# The following operators are defined:
#
#   operator    |   action
#   ------------|-------------
#      +, |     | union
#      -        | difference
#      *, &     | intersection
#       ~       | debug (see 03-debug-background.py)

d.save_as_scad()

# this is the same as 01-basics just using a more pythonic way to express it.
#
# If you execute this file it will create examples/02-vars-and-operators.py with
# the this content:
#
#    // Generated by ExpSolidPython
#
#    difference() {
#            cube(size = [10, 20, 30]);
#            sphere(r = 10);
#    };

